@@ -41,11 +41,12 @@
 #include "gibber-util.h"
 #include "gibber-transport.h"
 #include "gibber-fd-transport.h"
+#include "gibber-listener.h"
 
 #define DEBUG_FLAG DEBUG_BYTESTREAM
 #include "gibber-debug.h"
 
-#include "signals-marshal.h"
+#include "gibber-signals-marshal.h"
 
 static void
 bytestream_iface_init (gpointer g_iface, gpointer iface_data);
@@ -55,17 +56,6 @@ G_DEFINE_TYPE_WITH_CODE (GibberBytestreamOOB, gibber_bytestream_oob,
     G_IMPLEMENT_INTERFACE (GIBBER_TYPE_BYTESTREAM_IFACE,
       bytestream_iface_init));
 
-/* signals */
-enum
-{
-  DATA_RECEIVED,
-  STATE_CHANGED,
-  WRITE_BLOCKED,
-  LAST_SIGNAL
-};
-
-static guint signals[LAST_SIGNAL] = {0};
-
 /* properties */
 enum
 {
@@ -98,10 +88,9 @@ struct _GibberBytestreamIBBPrivate
    * If not we are the sender */
   gboolean recipient;
   GibberTransport *transport;
-  GIOChannel *listener;
-  guint listener_watch;
   gboolean write_blocked;
   gboolean read_blocked;
+  GibberListener *listener;
 
   GibberBytestreamOOBCheckAddrFunc check_addr_func;
   gpointer check_addr_func_data;
@@ -110,7 +99,7 @@ struct _GibberBytestreamIBBPrivate
 };
 
 #define GIBBER_BYTESTREAM_OOB_GET_PRIVATE(obj) \
-    ((GibberBytestreamOOBPrivate *) (GibberBytestreamOOB *)obj->priv)
+    ((GibberBytestreamOOBPrivate *) (GibberBytestreamOOB *) obj->priv)
 
 static void gibber_bytestream_oob_do_close (GibberBytestreamOOB *self,
     GError *error, gboolean can_wait);
@@ -149,7 +138,7 @@ transport_handler (GibberTransport *transport,
 
   buffer = g_string_new_len ((const gchar *) data->data, data->length);
 
-  g_signal_emit (G_OBJECT (self), signals[DATA_RECEIVED], 0, priv->peer_id,
+  g_signal_emit_by_name (G_OBJECT (self), "data-received", priv->peer_id,
       buffer);
 
   g_string_free (buffer, TRUE);
@@ -198,7 +187,7 @@ change_write_blocked_state (GibberBytestreamOOB *self,
     return;
 
   priv->write_blocked = blocked;
-  g_signal_emit (self, signals[WRITE_BLOCKED], 0, blocked);
+  g_signal_emit_by_name (self, "write-blocked", blocked);
 }
 
 static void
@@ -212,7 +201,6 @@ transport_buffer_empty_cb (GibberTransport *transport,
       DEBUG ("buffer is now empty. Bytestream can be closed");
       bytestream_closed (self);
     }
-
   else if (priv->write_blocked)
     {
       DEBUG ("buffer is empty, unblock write to the bytestream");
@@ -228,14 +216,23 @@ set_transport (GibberBytestreamOOB *self,
 
   g_assert (priv->transport == NULL);
 
-  priv->transport = transport;
+  priv->transport = g_object_ref (transport);
   gibber_transport_set_handler (transport, transport_handler, self);
 
+  /* The transport will already be connected if it is created from
+   * GibberListener. In this case, set the bytestream to open, otherwise
+   * it will be done in transport_connected_cb. */
+  if (gibber_transport_get_state (transport) == GIBBER_TRANSPORT_CONNECTED)
+    {
+      g_object_set (self, "state", GIBBER_BYTESTREAM_STATE_OPEN,
+          NULL);
+    }
+
   g_signal_connect (transport, "connected",
       G_CALLBACK (transport_connected_cb), self);
   g_signal_connect (transport, "disconnected",
       G_CALLBACK (transport_disconnected_cb), self);
-  g_signal_connect (priv->transport, "buffer-empty",
+  g_signal_connect (transport, "buffer-empty",
       G_CALLBACK (transport_buffer_empty_cb), self);
 }
 
@@ -303,6 +300,7 @@ connect_to_url (GibberBytestreamOOB *self,
   ll_transport = gibber_ll_transport_new ();
   set_transport (self, GIBBER_TRANSPORT (ll_transport));
   gibber_ll_transport_open_sockaddr (ll_transport, &addr, NULL);
+  g_object_unref (ll_transport);
 
 out:
   g_strfreev (tokens);
@@ -455,9 +453,8 @@ gibber_bytestream_oob_dispose (GObject *object)
 
   if (priv->listener != NULL)
     {
-      g_io_channel_unref (priv->listener);
+      g_object_unref (priv->listener);
       priv->listener = NULL;
-      g_source_remove (priv->listener_watch);
     }
 
   if (priv->xmpp_connection != NULL)
@@ -572,7 +569,7 @@ gibber_bytestream_oob_set_property (GObject *object,
         if (priv->state != g_value_get_uint (value))
             {
               priv->state = g_value_get_uint (value);
-              g_signal_emit (object, signals[STATE_CHANGED], 0, priv->state);
+              g_signal_emit_by_name (object, "state-changed", priv->state);
             }
         break;
       case PROP_HOST:
@@ -641,10 +638,7 @@ gibber_bytestream_oob_class_init (
       "bytestream if it's a private one",
       GIBBER_TYPE_XMPP_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_XMPP_CONNECTION,
       param_spec);
 
@@ -654,10 +648,7 @@ gibber_bytestream_oob_class_init (
       "the iq ID of the SI request, if any",
       "",
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STREAM_INIT_ID,
       param_spec);
 
@@ -667,39 +658,9 @@ gibber_bytestream_oob_class_init (
       "The hostname to use in the OOB URL. Literal are not allowed",
       "",
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_HOST,
       param_spec);
-
-  signals[DATA_RECEIVED] =
-    g_signal_new ("data-received",
-                  G_OBJECT_CLASS_TYPE (gibber_bytestream_oob_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  _gibber_signals_marshal_VOID__STRING_POINTER,
-                  G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_POINTER);
-
-  signals[STATE_CHANGED] =
-    g_signal_new ("state-changed",
-                  G_OBJECT_CLASS_TYPE (gibber_bytestream_oob_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__UINT,
-                  G_TYPE_NONE, 1, G_TYPE_UINT);
-
-  signals[WRITE_BLOCKED] =
-    g_signal_new ("write-blocked",
-                  G_OBJECT_CLASS_TYPE (gibber_bytestream_oob_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
-                  0,
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__BOOLEAN,
-                  G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
 }
 
 /*
@@ -725,8 +686,7 @@ gibber_bytestream_oob_send (GibberBytestreamIface *bytestream,
 
   if (priv->write_blocked)
     {
-      DEBUG ("can't send data for now, bytestream is blocked");
-      return FALSE;
+      DEBUG ("sending data while the bytestream was blocked");
     }
 
   DEBUG ("send %u bytes through bytestream", len);
@@ -953,144 +913,26 @@ make_oob_init_iq (const gchar *from,
       GIBBER_NODE_END, GIBBER_STANZA_END);
 }
 
-static gboolean
-listener_io_in_cb (GIOChannel *source,
-                   GIOCondition condition,
+static void
+new_connection_cb (GibberListener *listener,
+                   GibberTransport *transport,
+                   struct sockaddr *addr,
+                   guint size,
                    gpointer user_data)
 {
   GibberBytestreamOOB *self = GIBBER_BYTESTREAM_OOB (user_data);
   GibberBytestreamOOBPrivate *priv = GIBBER_BYTESTREAM_OOB_GET_PRIVATE (self);
-  int listen_fd, fd, ret;
-  char host[NI_MAXHOST];
-  char port[NI_MAXSERV];
-  struct sockaddr_storage addr;
-  socklen_t addrlen = sizeof (struct sockaddr_storage);
-  GibberLLTransport *ll_transport;
 
-  listen_fd = g_io_channel_unix_get_fd (source);
-  fd = accept (listen_fd, (struct sockaddr *) &addr, &addrlen);
-  gibber_normalize_address (&addr);
-
-  ret = getnameinfo ((struct sockaddr *) &addr, addrlen,
-      host, NI_MAXHOST, port, NI_MAXSERV,
-      NI_NUMERICHOST | NI_NUMERICSERV);
-
-  if (priv->check_addr_func != NULL && !priv->check_addr_func (self, &addr,
-        addrlen, priv->check_addr_func_data))
+  if (priv->check_addr_func != NULL && !priv->check_addr_func (self, addr,
+        size, priv->check_addr_func_data))
     {
-      DEBUG ("connection from %s refused by the bytestream user", host);
-      return TRUE;
+      DEBUG ("connection refused by the bytestream user");
+      return;
     }
 
-  if (ret == 0)
-    DEBUG("New connection from %s port %s", host, port);
-  else
-    DEBUG("New connection..");
+  DEBUG("New connection..");
 
-  ll_transport = gibber_ll_transport_new ();
-  set_transport (self, GIBBER_TRANSPORT (ll_transport));
-  gibber_ll_transport_open_fd (ll_transport, fd);
-
-  return FALSE;
-}
-
-static int
-start_listen_for_connection (GibberBytestreamOOB *self)
-{
-  GibberBytestreamOOBPrivate *priv = GIBBER_BYTESTREAM_OOB_GET_PRIVATE (self);
-  int port;
-  int fd = -1, ret, yes = 1;
-  struct addrinfo req, *ans = NULL;
-  struct sockaddr *addr;
-  struct sockaddr_in addr4;
-  struct sockaddr_in6 addr6;
-  socklen_t len;
-  #define BACKLOG 1
-
-  memset (&req, 0, sizeof (req));
-  req.ai_flags = AI_PASSIVE;
-  req.ai_family = AF_UNSPEC;
-  req.ai_socktype = SOCK_STREAM;
-  req.ai_protocol = IPPROTO_TCP;
-
-  ret = getaddrinfo (NULL, "0", &req, &ans);
-  if (ret != 0)
-    {
-      DEBUG ("getaddrinfo failed: %s", gai_strerror (ret));
-      goto error;
-    }
-
-  fd = socket (ans->ai_family, ans->ai_socktype, ans->ai_protocol);
-  if (fd == -1)
-    {
-      DEBUG ("socket failed: %s", g_strerror (errno));
-      goto error;
-    }
-
-  ret = setsockopt (fd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof (int));
-  if (ret == -1)
-    {
-      DEBUG ("setsockopt failed: %s", g_strerror (errno));
-      goto error;
-    }
-
-  ret = bind (fd, ans->ai_addr, ans->ai_addrlen);
-  if (ret  < 0)
-    {
-      DEBUG ("bind failed: %s", g_strerror (errno));
-      goto error;
-    }
-
-  if (ans->ai_family == AF_INET)
-    {
-      len = sizeof (struct sockaddr_in);
-      addr = (struct sockaddr *) &addr4;
-    }
-  else
-    {
-      len = sizeof (struct sockaddr_in6);
-      addr = (struct sockaddr *) &addr6;
-    }
-
-  if (getsockname (fd, addr, &len) == -1)
-  {
-    DEBUG ("getsockname failed: %s", g_strerror (errno));
-    goto error;
-  }
-
-  if (ans->ai_family == AF_INET)
-    {
-      port = ntohs (addr4.sin_port);
-    }
-  else
-    {
-      port = ntohs (addr6.sin6_port);
-    }
-
-  ret = listen (fd, BACKLOG);
-  if (ret == -1)
-    {
-      DEBUG ("listen failed: %s", g_strerror (errno));
-      goto error;
-    }
-
-  DEBUG ("listen on %s:%d", priv->host, port);
-
-  priv->listener = g_io_channel_unix_new (fd);
-  g_io_channel_set_close_on_unref (priv->listener, TRUE);
-  priv->listener_watch = g_io_add_watch (priv->listener, G_IO_IN,
-      listener_io_in_cb, self);
-
-  freeaddrinfo (ans);
-  return port;
-
-error:
-  if (fd > 0)
-    close (fd);
-
-  if (ans != NULL)
-    freeaddrinfo (ans);
-  return -1;
+  set_transport (self, transport);
 }
 
 /*
@@ -1098,7 +940,7 @@ error:
  *
  * Implements gibber_bytestream_iface_initiate on GibberBytestreamIface
  */
-gboolean
+static gboolean
 gibber_bytestream_oob_initiate (GibberBytestreamIface *bytestream)
 {
   GibberBytestreamOOB *self = GIBBER_BYTESTREAM_OOB (bytestream);
@@ -1119,12 +961,18 @@ gibber_bytestream_oob_initiate (GibberBytestreamIface *bytestream)
 
   priv->recipient = FALSE;
 
-  port = start_listen_for_connection (self);
-  if (port == -1)
+  g_assert (priv->listener == NULL);
+  priv->listener = gibber_listener_new ();
+
+  g_signal_connect (priv->listener, "new-connection",
+      G_CALLBACK (new_connection_cb), self);
+
+  if (!gibber_listener_listen_tcp (priv->listener, 0, NULL))
     {
       DEBUG ("can't listen for incoming connection");
       return FALSE;
     }
+  port = gibber_listener_get_port (priv->listener);
 
   url = g_strdup_printf ("x-tcp://%s:%d", priv->host, port);
 
@@ -1168,10 +1016,11 @@ gibber_bytestream_oob_set_check_addr_func (
   priv->check_addr_func_data = user_data;
 }
 
-void
-gibber_bytestream_oob_block_read (GibberBytestreamOOB *self,
-                                  gboolean block)
+static void
+gibber_bytestream_oob_block_reading (GibberBytestreamIface *bytestream,
+                                     gboolean block)
 {
+  GibberBytestreamOOB *self = GIBBER_BYTESTREAM_OOB (bytestream);
   GibberBytestreamOOBPrivate *priv = GIBBER_BYTESTREAM_OOB_GET_PRIVATE (self);
 
   if (priv->read_blocked == block)
@@ -1193,4 +1042,5 @@ bytestream_iface_init (gpointer g_iface,
   klass->send = gibber_bytestream_oob_send;
   klass->close = gibber_bytestream_oob_close;
   klass->accept = gibber_bytestream_oob_accept;
+  klass->block_reading = gibber_bytestream_oob_block_reading;
 }
