@@ -52,6 +52,17 @@ static void gibber_fd_transport_block_receiving (GibberTransport *transport,
 
 G_DEFINE_TYPE(GibberFdTransport, gibber_fd_transport, GIBBER_TYPE_TRANSPORT)
 
+GQuark
+gibber_fd_transport_error_quark (void)
+{
+  static GQuark quark = 0;
+
+  if (!quark)
+    quark = g_quark_from_static_string ("gibber_fd_transport_error");
+
+  return quark;
+}
+
 /* private structure */
 typedef struct _GibberFdTransportPrivate GibberFdTransportPrivate;
 
@@ -216,7 +227,7 @@ _writeout (GibberFdTransport *self, const guint8 *data, gsize len)
   GibberFdTransportPrivate *priv = GIBBER_FD_TRANSPORT_GET_PRIVATE (self);
   gsize written = 0;
 
-  DEBUG ("Writing out %zd bytes", len);
+  DEBUG ("Writing out %" G_GSIZE_FORMAT " bytes", len);
   if (priv->output_buffer == NULL || priv->output_buffer->len == 0)
     {
       /* We've got nothing buffer yet so try to write out directly */
@@ -234,12 +245,12 @@ _writeout (GibberFdTransport *self, const guint8 *data, gsize len)
 
   if (priv->output_buffer)
     {
-      g_string_append_len (priv->output_buffer, (gchar *)data + written,
+      g_string_append_len (priv->output_buffer, (gchar *) data + written,
         len - written);
     }
   else
     {
-      priv->output_buffer = g_string_new_len ((gchar *)data + written,
+      priv->output_buffer = g_string_new_len ((gchar *) data + written,
           len - written);
     }
 
@@ -287,7 +298,7 @@ _channel_io_out (GIOChannel *source, GIOCondition condition, gpointer data)
   gsize written;
 
   g_assert (priv->output_buffer);
-  if (!_try_write (self, (guint8 *)priv->output_buffer->str,
+  if (!_try_write (self, (guint8 *) priv->output_buffer->str,
                    priv->output_buffer->len, &written))
     {
       return FALSE;
@@ -311,8 +322,36 @@ _channel_io_out (GIOChannel *source, GIOCondition condition, gpointer data)
 static gboolean
 _channel_io_err (GIOChannel *source, GIOCondition condition, gpointer data)
 {
-  g_assert_not_reached ();
-  return TRUE;
+  GibberFdTransport *self = GIBBER_FD_TRANSPORT (data);
+  GError *error = NULL;
+  gint code;
+  const gchar *msg;
+
+  if (condition & G_IO_ERR)
+    {
+      DEBUG ("Error on GIOChannel.Closing the transport");
+      /* We can't use g_io_channel_error_from_errno because it seems errno is
+       * not always set when we got a G_IO_ERR. */
+      code = GIBBER_FD_TRANSPORT_ERROR_FAILED;
+      msg = "Error on GIOChannel";
+    }
+  else if (condition & G_IO_HUP)
+    {
+      DEBUG ("Connection has been broken. Closing the transport");
+      code = GIBBER_FD_TRANSPORT_ERROR_PIPE;
+      msg = "Connection has been broken";
+    }
+  else
+    {
+      g_assert_not_reached ();
+    }
+
+  error = g_error_new_literal (GIBBER_FD_TRANSPORT_ERROR, code, msg);
+  gibber_transport_emit_error (GIBBER_TRANSPORT (self), error);
+  g_error_free (error);
+
+  _do_disconnect (self);
+  return FALSE;
 }
 
 /* Default read and write implementations */
@@ -323,7 +362,7 @@ gibber_fd_transport_write (GibberFdTransport *fd_transport,
 {
   GIOStatus status;
 
-  status = g_io_channel_write_chars (channel, (gchar *)data, len,
+  status = g_io_channel_write_chars (channel, (gchar *) data, len,
     written, error);
 
   switch (status)
@@ -349,18 +388,18 @@ gibber_fd_transport_read (GibberFdTransport *transport,
 {
   guint8  buf[BUFSIZE + 1];
   GIOStatus status;
-  gsize read;
+  gsize bytes_read;
 
-  status = g_io_channel_read_chars (channel, (gchar *)buf, BUFSIZE,
-    &read, error);
+  status = g_io_channel_read_chars (channel, (gchar *) buf, BUFSIZE,
+    &bytes_read, error);
 
   switch (status)
     {
       case G_IO_STATUS_NORMAL:
-        buf[read] = '\0';
-        DEBUG ("Received %zd bytes", read);
+        buf[bytes_read] = '\0';
+        DEBUG ("Received %" G_GSIZE_FORMAT " bytes", bytes_read);
         gibber_transport_received_data (GIBBER_TRANSPORT (transport),
-            buf, read);
+            buf, bytes_read);
         return GIBBER_FD_IO_RESULT_SUCCESS;
       case G_IO_STATUS_ERROR:
         return GIBBER_FD_IO_RESULT_ERROR;
