@@ -8,10 +8,6 @@
 
 #include <gibber/gibber-xmpp-stanza.h>
 
-G_BEGIN_DECLS
-
-void debug_set_log_file_from_env (void);
-
 #ifdef ENABLE_DEBUG
 
 typedef enum
@@ -27,16 +23,17 @@ typedef enum
   DEBUG_MUC_CONNECTION = 1 << 8,
   DEBUG_CONNECTION     = 1 << 9,
   DEBUG_IM             = 1 << 10,
-  DEBUG_BYTESTREAM_MGR = 1 << 11,
+  DEBUG_SI_BYTESTREAM_MGR = 1 << 11,
   DEBUG_NET            = 1 << 12,
   DEBUG_SELF           = 1 << 13,
   DEBUG_TUBES          = 1 << 14,
   DEBUG_XCM            = 1 << 15,
   DEBUG_DISCOVERY      = 1 << 16,
   DEBUG_OLPC_ACTIVITY  = 1 << 17,
+  DEBUG_FT             = 1 << 18,
 } DebugFlags;
 
-void debug_set_flags_from_env ();
+void debug_set_flags_from_env (void);
 void debug_set_flags (DebugFlags flags);
 gboolean debug_flag_is_set (DebugFlags flag);
 void debug (DebugFlags flag, const gchar *format, ...)
@@ -55,11 +52,13 @@ void debug (DebugFlags flag, const gchar *format, ...)
 
 #ifdef DEBUG_FLAG
 
-#define DEBUG(format, ...)
+#define DEBUG(format, ...) \
+  G_STMT_START { } G_STMT_END
 
 #define DEBUGGING 0
 
-#define NODE_DEBUG(n, s)
+#define NODE_DEBUG(n, s) \
+  G_STMT_START { } G_STMT_END
 
 #endif /* DEBUG_FLAG */
 
