@@ -88,7 +88,7 @@ struct _SalutAvahiContactPrivate
 };
 
 #define SALUT_AVAHI_CONTACT_GET_PRIVATE(obj) \
-    ((SalutAvahiContactPrivate *) ((SalutAvahiContact *)obj)->priv)
+    ((SalutAvahiContactPrivate *) ((SalutAvahiContact *) obj)->priv)
 
 static void
 salut_avahi_contact_init (SalutAvahiContact *self)
@@ -145,8 +145,8 @@ salut_avahi_contact_set_property (GObject *object,
 static void
 _avahi_address_to_sockaddr (AvahiAddress *address,
                             guint16 port,
-                            AvahiIfIndex index,
-                            struct sockaddr_storage *sockaddr)
+                            AvahiIfIndex index_,
+                            struct sockaddr *sockaddr)
 {
   switch (address->proto)
     {
@@ -167,7 +167,7 @@ _avahi_address_to_sockaddr (AvahiAddress *address,
           memcpy (sockaddr6->sin6_addr.s6_addr, address->data.ipv6.address, 16);
 
           sockaddr6->sin6_flowinfo = 0;
-          sockaddr6->sin6_scope_id = index;
+          sockaddr6->sin6_scope_id = index_;
           break;
        }
     default:
@@ -188,7 +188,7 @@ _add_address (GaServiceResolver *resolver,
   if (ga_service_resolver_get_address (resolver, &address, &port))
     {
       _avahi_address_to_sockaddr (&address, port, ifindex,
-          &(s_address.address));
+          (struct sockaddr *) &s_address.address);
       g_array_append_val (addresses, s_address);
     }
 }
@@ -209,7 +209,7 @@ salut_avahi_contact_get_addresses (SalutContact *contact)
 
 static gint
 _compare_address (GaServiceResolver *resolver,
-                  struct sockaddr_storage *addr_b)
+                  struct sockaddr *addr_b)
 {
   struct sockaddr_storage addr_a;
   AvahiIfIndex ifindex;
@@ -220,13 +220,13 @@ _compare_address (GaServiceResolver *resolver,
   if (!ga_service_resolver_get_address (resolver, &address, &port))
     return -1;
 
-  _avahi_address_to_sockaddr (&address, port, ifindex, &addr_a);
+  _avahi_address_to_sockaddr (&address, port, ifindex,
+      (struct sockaddr *) &addr_a);
 
-  if ( ((struct sockaddr *) &addr_a)->sa_family
-       != ((struct sockaddr *) addr_b)->sa_family)
+  if (addr_a.ss_family != addr_b->sa_family)
     return -1;
 
-  switch (((struct sockaddr *) &addr_a)->sa_family)
+  switch (addr_a.ss_family)
     {
       case AF_INET:
         {
@@ -250,7 +250,8 @@ _compare_address (GaServiceResolver *resolver,
 
 static gboolean
 salut_avahi_contact_has_address (SalutContact *contact,
-                                 struct sockaddr_storage *address)
+                                 struct sockaddr *address,
+                                 guint size)
 {
   SalutAvahiContact *self = SALUT_AVAHI_CONTACT (contact);
   SalutAvahiContactPrivate *priv = SALUT_AVAHI_CONTACT_GET_PRIVATE (self);
@@ -305,8 +306,8 @@ salut_contact_avatar_found (GaRecordBrowser *browser,
                             AvahiLookupFlags flags,
                             SalutAvahiContact *self)
 {
-  DEBUG ("Found avatar for %s for size %zd", SALUT_CONTACT (self)->name,
-      rdata_size);
+  DEBUG ("Found avatar for %s for size %" G_GSIZE_FORMAT,
+      SALUT_CONTACT (self)->name, rdata_size);
 
   if (rdata_size <= 0)
     salut_avahi_contact_avatar_request_flush (self, NULL, 0);
@@ -384,10 +385,7 @@ salut_avahi_contact_class_init (
       "The Salut Avahi Discovery client associated with this muc manager",
       SALUT_TYPE_AVAHI_DISCOVERY_CLIENT,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CLIENT,
       param_spec);
 }
@@ -687,7 +685,7 @@ contact_resolved_cb (GaServiceResolver *resolver,
   if (olpc_key_part != NULL)
     {
       guint i = 0;
-      gchar *name = NULL;
+      gchar *olpc_key_part_name = NULL;
       GArray *olpc_key;
 
       /* FIXME: how big are OLPC keys anyway? */
@@ -699,10 +697,10 @@ contact_resolved_cb (GaServiceResolver *resolver,
           avahi_free (olpc_key_part);
 
           i++;
-          name = g_strdup_printf ("olpc-key-part%u", i);
-          olpc_key_part = _avahi_txt_get_keyval_with_size (txt, name,
-              &size);
-          g_free (name);
+          olpc_key_part_name = g_strdup_printf ("olpc-key-part%u", i);
+          olpc_key_part = _avahi_txt_get_keyval_with_size (txt,
+              olpc_key_part_name, &size);
+          g_free (olpc_key_part_name);
         }
       while (olpc_key_part != NULL);
 
