@@ -28,6 +28,23 @@
 #include "salut-connection-manager.h"
 #include "salut-connection.h"
 
+/* properties */
+enum
+{
+  PROP_BACKEND = 1,
+  LAST_PROPERTY
+};
+
+typedef struct _SalutConnectionManagerPrivate SalutConnectionManagerPrivate;
+struct _SalutConnectionManagerPrivate
+{
+  GType backend_type;
+};
+
+#define SALUT_CONNECTION_MANAGER_GET_PRIVATE(obj) \
+    ((SalutConnectionManagerPrivate *) ((SalutConnectionManager *) obj)->priv)
+
+
 typedef struct {
   guint set_mask;
 
@@ -90,16 +107,79 @@ G_DEFINE_TYPE(SalutConnectionManager, salut_connection_manager,
               TP_TYPE_BASE_CONNECTION_MANAGER)
 
 static void
-salut_connection_manager_init (SalutConnectionManager *obj)
+salut_connection_manager_init (SalutConnectionManager *self)
 {
+  SalutConnectionManagerPrivate *priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      SALUT_TYPE_CONNECTION_MANAGER, SalutConnectionManagerPrivate);
+
+  self->priv = priv;
+}
+
+static void
+salut_connection_manager_get_property (GObject *object,
+                                       guint property_id,
+                                       GValue *value,
+                                       GParamSpec *pspec)
+{
+  SalutConnectionManager *self = SALUT_CONNECTION_MANAGER (object);
+  SalutConnectionManagerPrivate *priv = SALUT_CONNECTION_MANAGER_GET_PRIVATE
+      (self);
+
+  switch (property_id)
+    {
+      case PROP_BACKEND:
+        g_value_set_gtype (value, priv->backend_type);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+salut_connection_manager_set_property (GObject *object,
+                                       guint property_id,
+                                       const GValue *value,
+                                       GParamSpec *pspec)
+{
+  SalutConnectionManager *self = SALUT_CONNECTION_MANAGER (object);
+  SalutConnectionManagerPrivate *priv = SALUT_CONNECTION_MANAGER_GET_PRIVATE
+      (self);
+
+  switch (property_id)
+    {
+      case PROP_BACKEND:
+        priv->backend_type = g_value_get_gtype (value);
+        break;
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
 }
 
 static void
 salut_connection_manager_class_init (
     SalutConnectionManagerClass *salut_connection_manager_class)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (salut_connection_manager_class);
   TpBaseConnectionManagerClass *base_cm_class =
     TP_BASE_CONNECTION_MANAGER_CLASS(salut_connection_manager_class);
+  GParamSpec *param_spec;
+
+  g_type_class_add_private (salut_connection_manager_class,
+      sizeof (SalutConnectionManagerPrivate));
+
+  object_class->get_property = salut_connection_manager_get_property;
+  object_class->set_property = salut_connection_manager_set_property;
+
+  param_spec = g_param_spec_gtype (
+      "backend-type",
+      "backend type",
+      "a G_TYPE_GTYPE of the backend to use",
+      G_TYPE_NONE,
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_BACKEND,
+      param_spec);
 
   base_cm_class->cm_dbus_name = "salut";
   base_cm_class->protocol_params = salut_protocols;
@@ -113,7 +193,7 @@ static void *salut_params_new (void) {
 
 static void salut_params_free (void *params)
 {
-  SalutParams *p = (SalutParams *)params;
+  SalutParams *p = (SalutParams *) params;
 
   g_free (p->nickname);
   g_free (p->first_name);
@@ -138,12 +218,17 @@ salut_connection_manager_new_connection (TpBaseConnectionManager *self,
                                          void *parsed_params,
                                          GError **error)
 {
+  SalutConnectionManagerPrivate *priv = SALUT_CONNECTION_MANAGER_GET_PRIVATE
+      (self);
   SalutConnection *conn;
-  SalutParams *params = (SalutParams *)parsed_params;
+  SalutParams *params = (SalutParams *) parsed_params;
 
   g_assert (!tp_strdiff (proto, "local-xmpp"));
 
-  conn = g_object_new (SALUT_TYPE_CONNECTION, "protocol", proto, NULL);
+  conn = g_object_new (SALUT_TYPE_CONNECTION,
+      "protocol", proto,
+      "backend-type", priv->backend_type,
+      NULL);
 
   SET_PROPERTY_IF_PARAM_SET ("nickname", SALUT_PARAM_NICKNAME,
                               params->nickname);
