@@ -37,29 +37,33 @@
 #include "salut-contact-channel.h"
 #include "salut-im-manager.h"
 #include "salut-muc-manager.h"
+#include "salut-ft-manager.h"
 #include "salut-contact.h"
+#include "salut-roomlist-manager.h"
 #include "salut-self.h"
 #include "salut-xmpp-connection-manager.h"
-#include "salut-bytestream-manager.h"
+#include "salut-si-bytestream-manager.h"
 
 #ifdef ENABLE_OLPC
 #include "salut-olpc-activity-manager.h"
 #endif
 
-/*
 #include "salut-tubes-manager.h"
-*/
 
 #include "salut-presence.h"
 #include "salut-discovery-client.h"
 #include "salut-avahi-discovery-client.h"
 
+#include <extensions/extensions.h>
+
+#include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/handle-repo-dynamic.h>
 #include <telepathy-glib/handle-repo-static.h>
 #include <telepathy-glib/handle-repo.h>
 #include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/svc-generic.h>
 
 #include <gibber/gibber-namespaces.h>
 
@@ -75,8 +79,6 @@
   (dbus_g_type_get_struct ("GValueArray", G_TYPE_STRING, G_TYPE_UINT, \
       G_TYPE_INVALID))
 
-#include <extensions/extensions.h>
-
 static void
 salut_connection_olpc_buddy_info_iface_init (gpointer g_iface,
     gpointer iface_data);
@@ -102,6 +104,12 @@ G_DEFINE_TYPE_WITH_CODE(SalutConnection,
         salut_connection_aliasing_service_iface_init);
     G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_PRESENCE,
        tp_presence_mixin_iface_init);
+    G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_DBUS_PROPERTIES,
+       tp_dbus_properties_mixin_iface_init);
+    G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_CONTACTS,
+       tp_contacts_mixin_iface_init);
+    G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_SIMPLE_PRESENCE,
+       tp_presence_mixin_simple_presence_iface_init);
     G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_AVATARS,
        salut_connection_avatar_service_iface_init);
 #ifdef ENABLE_OLPC
@@ -132,13 +140,16 @@ enum {
   PROP_PUBLISHED_NAME,
   PROP_IM_MANAGER,
   PROP_MUC_MANAGER,
+  PROP_TUBES_MANAGER,
+  PROP_ROOMLIST_MANAGER,
   PROP_CONTACT_MANAGER,
   PROP_SELF,
   PROP_XCM,
-  PROP_BYTESTREAM_MANAGER,
+  PROP_SI_BYTESTREAM_MANAGER,
 #ifdef ENABLE_OLPC
   PROP_OLPC_ACTIVITY_MANAGER,
 #endif
+  PROP_BACKEND,
   LAST_PROP
 };
 
@@ -178,20 +189,28 @@ struct _SalutConnectionPrivate
   /* MUC channel manager */
   SalutMucManager *muc_manager;
 
-  /* Tubes channel manager */
-  /* XXX disabled while private tubes aren't implemented */
-  /* SalutTubesManager *tubes_manager; */
+  /* FT channel manager */
+  SalutFtManager *ft_manager;
 
-  /* Bytestream manager */
-  SalutBytestreamManager *bytestream_manager;
+  /* Roomlist channel manager */
+  SalutRoomlistManager *roomlist_manager;
+
+  /* Tubes channel manager */
+  SalutTubesManager *tubes_manager;
+
+  /* Bytestream manager for stream initiation (XEP-0095) */
+  SalutSiBytestreamManager *si_bytestream_manager;
 
 #ifdef ENABLE_OLPC
   SalutOlpcActivityManager *olpc_activity_manager;
 #endif
+
+  /* Backend type: avahi or dummy */
+  GType backend_type;
 };
 
 #define SALUT_CONNECTION_GET_PRIVATE(o) \
-  ((SalutConnectionPrivate *)((SalutConnection *)o)->priv);
+  ((SalutConnectionPrivate *)((SalutConnection *) o)->priv)
 
 typedef struct _ChannelRequest ChannelRequest;
 
@@ -213,6 +232,9 @@ salut_connection_create_handle_repos (TpBaseConnection *self,
 static GPtrArray *
 salut_connection_create_channel_factories (TpBaseConnection *self);
 
+static GPtrArray *
+salut_connection_create_channel_managers (TpBaseConnection *self);
+
 static gchar *
 salut_connection_get_unique_connection_name (TpBaseConnection *self);
 
@@ -222,6 +244,12 @@ salut_connection_shut_down (TpBaseConnection *self);
 static gboolean
 salut_connection_start_connecting (TpBaseConnection *self, GError **error);
 
+static void salut_connection_avatars_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash);
+
+static void salut_connection_aliasing_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash);
+
 static void
 salut_connection_init (SalutConnection *obj)
 {
@@ -252,10 +280,33 @@ salut_connection_init (SalutConnection *obj)
 
   priv->contact_manager = NULL;
   priv->xmpp_connection_manager = NULL;
+}
 
-  /* FIXME: make this configurable */
-  priv->discovery_client = SALUT_DISCOVERY_CLIENT (
-      salut_avahi_discovery_client_new ());
+static GObject *
+salut_connection_constructor (GType type,
+                              guint n_props,
+                              GObjectConstructParam *props)
+{
+  GObject *obj;
+
+  obj = G_OBJECT_CLASS (salut_connection_parent_class)->
+           constructor (type, n_props, props);
+
+  tp_contacts_mixin_init (obj,
+      G_STRUCT_OFFSET (SalutConnection, contacts_mixin));
+
+  tp_base_connection_register_with_contacts_mixin (TP_BASE_CONNECTION (obj));
+  tp_presence_mixin_simple_presence_register_with_contacts_mixin (obj);
+
+  tp_contacts_mixin_add_contact_attributes_iface (obj,
+      TP_IFACE_CONNECTION_INTERFACE_AVATARS,
+      salut_connection_avatars_fill_contact_attributes);
+
+  tp_contacts_mixin_add_contact_attributes_iface (obj,
+      TP_IFACE_CONNECTION_INTERFACE_ALIASING,
+      salut_connection_aliasing_fill_contact_attributes);
+
+  return obj;
 }
 
 static void
@@ -293,6 +344,12 @@ salut_connection_get_property (GObject *object,
     case PROP_MUC_MANAGER:
       g_value_set_object (value, priv->muc_manager);
       break;
+    case PROP_TUBES_MANAGER:
+      g_value_set_object (value, priv->tubes_manager);
+      break;
+    case PROP_ROOMLIST_MANAGER:
+      g_value_set_object (value, priv->roomlist_manager);
+      break;
     case PROP_CONTACT_MANAGER:
       g_value_set_object (value, priv->contact_manager);
       break;
@@ -302,14 +359,17 @@ salut_connection_get_property (GObject *object,
     case PROP_XCM:
       g_value_set_object (value, priv->xmpp_connection_manager);
       break;
-    case PROP_BYTESTREAM_MANAGER:
-      g_value_set_object (value, priv->bytestream_manager);
+    case PROP_SI_BYTESTREAM_MANAGER:
+      g_value_set_object (value, priv->si_bytestream_manager);
       break;
 #ifdef ENABLE_OLPC
     case PROP_OLPC_ACTIVITY_MANAGER:
       g_value_set_object (value, priv->olpc_activity_manager);
       break;
 #endif
+    case PROP_BACKEND:
+      g_value_set_gtype (value, priv->backend_type);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -350,6 +410,13 @@ salut_connection_set_property (GObject *object,
       g_free (priv->published_name);
       priv->published_name = g_value_dup_string (value);
       break;
+    case PROP_BACKEND:
+      priv->backend_type = g_value_get_gtype (value);
+      /* Create the backend object */
+      priv->discovery_client = g_object_new (priv->backend_type,
+          NULL);
+      g_assert (priv->discovery_client != NULL);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -387,9 +454,9 @@ const char *salut_presence_status_txt_names[] = {
 
 static gboolean
 is_presence_status_available (GObject *obj,
-                              guint index)
+                              guint index_)
 {
-  return (index >= 0 && index < SALUT_PRESENCE_NR_PRESENCES);
+  return (index_ < SALUT_PRESENCE_OFFLINE);
 }
 
 static GHashTable *
@@ -523,7 +590,9 @@ set_own_status (GObject *obj,
     }
   else
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, err->message);
+      if (error != NULL)
+        *error = g_error_new_literal (TP_ERRORS, TP_ERROR_NETWORK_ERROR,
+            err->message);
     }
 
   return TRUE;
@@ -538,8 +607,11 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
   GParamSpec *param_spec;
   static const gchar *interfaces [] = {
     TP_IFACE_CONNECTION_INTERFACE_ALIASING,
-    TP_IFACE_CONNECTION_INTERFACE_PRESENCE,
     TP_IFACE_CONNECTION_INTERFACE_AVATARS,
+    TP_IFACE_CONNECTION_INTERFACE_CONTACTS,
+    TP_IFACE_CONNECTION_INTERFACE_PRESENCE,
+    TP_IFACE_CONNECTION_INTERFACE_SIMPLE_PRESENCE,
+    TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
 #ifdef ENABLE_OLPC
     SALUT_IFACE_OLPC_BUDDY_INFO,
     SALUT_IFACE_OLPC_ACTIVITY_PROPERTIES,
@@ -552,6 +624,8 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
   g_type_class_add_private (salut_connection_class,
       sizeof (SalutConnectionPrivate));
 
+  object_class->constructor = salut_connection_constructor;
+
   object_class->dispose = salut_connection_dispose;
   object_class->finalize = salut_connection_finalize;
 
@@ -559,6 +633,8 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       salut_connection_create_handle_repos;
   tp_connection_class->create_channel_factories =
       salut_connection_create_channel_factories;
+  tp_connection_class->create_channel_managers =
+      salut_connection_create_channel_managers;
   tp_connection_class->get_unique_connection_name =
       salut_connection_get_unique_connection_name;
   tp_connection_class->shut_down =
@@ -567,39 +643,48 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       salut_connection_start_connecting;
   tp_connection_class->interfaces_always_present = interfaces;
 
+  salut_connection_class->properties_mixin.interfaces = NULL;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (SalutConnectionClass, properties_mixin));
+
   tp_presence_mixin_class_init (object_class,
       G_STRUCT_OFFSET (SalutConnectionClass, presence_mixin),
       is_presence_status_available, get_contact_statuses, set_own_status,
       presence_statuses);
 
+  tp_presence_mixin_simple_presence_init_dbus_properties (object_class);
+
+  tp_contacts_mixin_class_init (object_class,
+        G_STRUCT_OFFSET (SalutConnectionClass, contacts_mixin));
+
   param_spec = g_param_spec_string ("nickname", "nickname",
       "Nickname used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_NICKNAME, param_spec);
 
   param_spec = g_param_spec_string ("first-name", "First name",
       "First name used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_FIRST_NAME, param_spec);
 
   param_spec = g_param_spec_string ("last-name", "Last name",
       "Last name used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_LAST_NAME, param_spec);
 
   param_spec = g_param_spec_string ("email", "E-mail address",
       "E-mail address used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_EMAIL, param_spec);
 
   param_spec = g_param_spec_string ("jid", "Jabber id",
       "Jabber id used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_JID, param_spec);
 
   param_spec = g_param_spec_string ("published-name", "Published name",
       "Username used in the published data", NULL,
-      G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_PUBLISHED_NAME,
       param_spec);
 
@@ -608,9 +693,7 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       "SalutImManager object",
       "The Salut IM Manager associated with this Salut Connection",
       SALUT_TYPE_IM_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_IM_MANAGER,
       param_spec);
 
@@ -619,20 +702,34 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       "SalutMucManager object",
       "The Salut MUC Manager associated with this Salut Connection",
       SALUT_TYPE_MUC_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_MUC_MANAGER,
       param_spec);
 
+  param_spec = g_param_spec_object (
+      "tubes-manager",
+      "SalutTubesManager object",
+      "The Salut Tubes Manager associated with this Salut Connection",
+      SALUT_TYPE_TUBES_MANAGER,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_TUBES_MANAGER,
+      param_spec);
+
+  param_spec = g_param_spec_object (
+      "roomlist-manager",
+      "SalutRoomlistManager object",
+      "The Salut Roomlist Manager associated with this Salut Connection",
+      SALUT_TYPE_ROOMLIST_MANAGER,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_ROOMLIST_MANAGER,
+      param_spec);
+
   param_spec = g_param_spec_object (
       "contact-manager",
       "SalutContactManager object",
       "The Salut Contact Manager associated with this Salut Connection",
       SALUT_TYPE_CONTACT_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTACT_MANAGER,
       param_spec);
 
@@ -641,9 +738,7 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       "SalutSelf object",
       "The Salut Self object associated with this Salut Connection",
       SALUT_TYPE_SELF,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_SELF,
       param_spec);
 
@@ -653,21 +748,17 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       "The Salut XMPP Connection Manager associated with this Salut "
       "Connection",
       SALUT_TYPE_XMPP_CONNECTION_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_XCM,
       param_spec);
 
   param_spec = g_param_spec_object (
-      "bytestream-manager",
-      "SalutBytestreamManager object",
-      "The Salut Bytestream Manager associated with this Salut Connection",
-      SALUT_TYPE_BYTESTREAM_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_BYTESTREAM_MANAGER,
+      "si-bytestream-manager",
+      "SalutSiBytestreamManager object",
+      "The Salut SI Bytestream Manager associated with this Salut Connection",
+      SALUT_TYPE_SI_BYTESTREAM_MANAGER,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_SI_BYTESTREAM_MANAGER,
       param_spec);
 
 #ifdef ENABLE_OLPC
@@ -676,12 +767,21 @@ salut_connection_class_init (SalutConnectionClass *salut_connection_class)
       "SalutOlpcActivityManager object",
       "The OLPC activity Manager associated with this Salut Connection",
       SALUT_TYPE_OLPC_ACTIVITY_MANAGER,
-      G_PARAM_READABLE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_OLPC_ACTIVITY_MANAGER,
       param_spec);
 #endif
+
+  param_spec = g_param_spec_gtype (
+      "backend-type",
+      "backend type",
+      "a G_TYPE_GTYPE of the backend to use",
+      G_TYPE_NONE,
+      G_PARAM_CONSTRUCT_ONLY |
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_BACKEND,
+      param_spec);
+
 }
 
 void
@@ -726,10 +826,10 @@ salut_connection_dispose (GObject *object)
       priv->discovery_client = NULL;
     }
 
-  if (priv->bytestream_manager != NULL)
+  if (priv->si_bytestream_manager != NULL)
     {
-      g_object_unref (priv->bytestream_manager);
-      priv->bytestream_manager = NULL;
+      g_object_unref (priv->si_bytestream_manager);
+      priv->si_bytestream_manager = NULL;
     }
 
   /* release any references held by the object here */
@@ -757,12 +857,14 @@ salut_connection_finalize (GObject *object)
   g_free (priv->olpc_color);
 #endif
 
+  tp_contacts_mixin_finalize (G_OBJECT(self));
+
   DEBUG("Finalizing connection");
 
   G_OBJECT_CLASS (salut_connection_parent_class)->finalize (object);
 }
 
-void
+static void
 _contact_manager_contact_status_changed (SalutConnection *self,
     SalutContact *contact, TpHandle handle)
 {
@@ -793,23 +895,26 @@ _self_established_cb (SalutSelf *s, gpointer data)
 
   if (!salut_contact_manager_start (priv->contact_manager, NULL))
     {
-      /* FIXME handle error */
       tp_base_connection_change_status ( TP_BASE_CONNECTION (base),
-          TP_CONNECTION_STATUS_CONNECTING,
-          TP_CONNECTION_STATUS_REASON_REQUESTED);
+          TP_CONNECTION_STATUS_DISCONNECTED,
+          TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
       return;
   }
 
-  if (!salut_muc_manager_start (priv->muc_manager, NULL))
+  if (!salut_roomlist_manager_start (priv->roomlist_manager, NULL))
     {
-      /* XXX handle error */
+      tp_base_connection_change_status ( TP_BASE_CONNECTION (base),
+          TP_CONNECTION_STATUS_DISCONNECTED,
+          TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
       return;
     }
 
 #ifdef ENABLE_OLPC
   if (!salut_olpc_activity_manager_start (priv->olpc_activity_manager, NULL))
     {
-      /* XXX handle error */
+      tp_base_connection_change_status ( TP_BASE_CONNECTION (base),
+          TP_CONNECTION_STATUS_DISCONNECTED,
+          TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
       return;
     }
 #endif
@@ -864,10 +969,8 @@ discovery_client_running (SalutConnection *self)
     }
 
   /* Create the bytestream manager */
-  /* FIXME: we should abstract this */
-  priv->bytestream_manager = salut_bytestream_manager_new (self,
-    avahi_client_get_host_name_fqdn (SALUT_AVAHI_DISCOVERY_CLIENT (
-        priv->discovery_client)->avahi_client->avahi_client));
+  priv->si_bytestream_manager = salut_si_bytestream_manager_new (self,
+    salut_discovery_client_get_host_name_fqdn (priv->discovery_client));
 }
 
 static void
@@ -922,7 +1025,7 @@ _salut_connection_disconnect (SalutConnection *self)
  * on interface org.freedesktop.Telepathy.Connection.Interface.Aliasing
  *
  */
-void
+static void
 salut_connection_get_alias_flags (TpSvcConnectionInterfaceAliasing *self,
     DBusGMethodInvocation *context)
 {
@@ -933,6 +1036,39 @@ salut_connection_get_alias_flags (TpSvcConnectionInterfaceAliasing *self,
       0);
 }
 
+static const gchar *
+salut_connection_get_alias (SalutConnection *self, TpHandle handle)
+{
+  SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
+  TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (base,
+    TP_HANDLE_TYPE_CONTACT);
+  const gchar *alias;
+
+  if (handle == base->self_handle)
+    {
+      alias = salut_self_get_alias (priv->self);
+    }
+  else
+    {
+      SalutContact *contact;
+      contact = salut_contact_manager_get_contact (priv->contact_manager,
+        handle);
+
+      if (contact == NULL)
+        {
+          alias = tp_handle_inspect (contact_repo, handle);
+        }
+      else
+        {
+          alias = salut_contact_get_alias (contact);
+          g_object_unref (contact);
+        }
+    }
+
+  return alias;
+}
+
 /**
  * salut_connection_request_aliases
  *
@@ -940,14 +1076,13 @@ salut_connection_get_alias_flags (TpSvcConnectionInterfaceAliasing *self,
  * on interface org.freedesktop.Telepathy.Connection.Interface.Aliasing
  *
  */
-void
+static void
 salut_connection_request_aliases (TpSvcConnectionInterfaceAliasing *iface,
     const GArray *contacts, DBusGMethodInvocation *context)
 {
   SalutConnection *self = SALUT_CONNECTION (iface);
-  SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
   TpBaseConnection *base = TP_BASE_CONNECTION (self);
-  int i;
+  guint i;
   const gchar **aliases;
   GError *error = NULL;
   TpHandleRepoIface *contact_handles =
@@ -968,26 +1103,8 @@ salut_connection_request_aliases (TpSvcConnectionInterfaceAliasing *iface,
   for (i = 0; i < contacts->len; i++)
     {
       TpHandle handle = g_array_index (contacts, TpHandle, i);
-      SalutContact *contact;
-      if (handle == TP_BASE_CONNECTION (self)->self_handle)
-        {
-          aliases[i] = salut_self_get_alias (priv->self);
-        }
-      else
-        {
-          contact = salut_contact_manager_get_contact (priv->contact_manager,
-            handle);
-          if (contact == NULL)
-            {
-              DEBUG ("RequestAliases called for offline contact");
-              aliases[i] = "";
-            }
-          else
-            {
-              aliases[i] = salut_contact_get_alias (contact);
-              g_object_unref (contact);
-          }
-        }
+
+      aliases[i] = salut_connection_get_alias (self, handle);
     }
 
   tp_svc_connection_interface_aliasing_return_from_request_aliases (context,
@@ -997,6 +1114,59 @@ salut_connection_request_aliases (TpSvcConnectionInterfaceAliasing *iface,
   return;
 }
 
+static void
+salut_connection_get_aliases (TpSvcConnectionInterfaceAliasing *iface,
+    const GArray *contacts, DBusGMethodInvocation *context)
+{
+  SalutConnection *self = SALUT_CONNECTION (iface);
+  TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (base,
+    TP_HANDLE_TYPE_CONTACT);
+  guint i;
+  GError *error = NULL;
+  GHashTable *result = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+    NULL, NULL);
+
+  if (!tp_handles_are_valid (contact_repo, contacts, FALSE, &error))
+    {
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
+    }
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+
+      g_hash_table_insert (result, GUINT_TO_POINTER (handle),
+        (gchar *) salut_connection_get_alias (self, handle));
+    }
+
+   tp_svc_connection_interface_aliasing_return_from_get_aliases (context,
+       result);
+
+   g_hash_table_destroy (result);
+}
+
+static void
+salut_connection_aliasing_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash)
+{
+  SalutConnection *self = SALUT_CONNECTION (obj);
+  guint i;
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+      GValue *val = tp_g_value_slice_new (G_TYPE_STRING);
+
+      g_value_set_string (val, salut_connection_get_alias (self, handle));
+
+      tp_contacts_mixin_set_contact_attribute (attributes_hash, handle,
+         TP_IFACE_CONNECTION_INTERFACE_ALIASING"/alias", val);
+    }
+}
+
 static void
 salut_connection_set_aliases (TpSvcConnectionInterfaceAliasing *iface,
     GHashTable *aliases, DBusGMethodInvocation *context)
@@ -1030,7 +1200,7 @@ salut_connection_set_aliases (TpSvcConnectionInterfaceAliasing *iface,
   tp_svc_connection_interface_aliasing_return_from_set_aliases (context);
 }
 
-void
+static void
 _contact_manager_contact_alias_changed  (SalutConnection *self,
     SalutContact *contact, TpHandle handle)
 {
@@ -1065,12 +1235,13 @@ salut_connection_aliasing_service_iface_init (gpointer g_iface,
     (klass, salut_connection_##x)
   IMPLEMENT (get_alias_flags);
   IMPLEMENT (request_aliases);
+  IMPLEMENT (get_aliases);
   IMPLEMENT (set_aliases);
 #undef IMPLEMENT
 }
 
 /* Avatar service implementation */
-void
+static void
 _contact_manager_contact_avatar_changed (SalutConnection *self,
     SalutContact *contact, TpHandle handle)
 {
@@ -1110,8 +1281,8 @@ salut_connection_set_avatar (TpSvcConnectionInterfaceAvatars *iface,
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
 
-  if (!salut_self_set_avatar (priv->self, (guint8 *)avatar->data,
-                             avatar->len, &error))
+  if (!salut_self_set_avatar (priv->self, (guint8 *) avatar->data,
+        avatar->len, &error))
     {
       dbus_g_method_return_error (context, error);
       g_error_free (error);
@@ -1129,16 +1300,17 @@ static void
 salut_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
     const GArray *contacts, DBusGMethodInvocation *context)
 {
-  int i;
+  guint i;
   gchar **ret;
   GError *err = NULL;
   SalutConnection *self = SALUT_CONNECTION (iface);
   SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
   TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  TpHandleRepoIface *handle_repo;
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
 
-  TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (base,
+  handle_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
   if (!tp_handles_are_valid (handle_repo, contacts, FALSE, &err))
@@ -1160,6 +1332,7 @@ salut_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
       else
         {
            SalutContact *contact;
+
            contact = salut_contact_manager_get_contact (priv->contact_manager,
                handle);
            if (contact != NULL)
@@ -1183,16 +1356,17 @@ salut_connection_get_known_avatar_tokens (
     TpSvcConnectionInterfaceAvatars *iface, const GArray *contacts,
     DBusGMethodInvocation *context)
 {
-  int i;
+  guint i;
   GHashTable *ret;
   GError *err = NULL;
   SalutConnection *self = SALUT_CONNECTION (iface);
   SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
   TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  TpHandleRepoIface *handle_repo;
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
 
-  TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (base,
+  handle_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
   if (!tp_handles_are_valid (handle_repo, contacts, FALSE, &err))
@@ -1239,6 +1413,53 @@ salut_connection_get_known_avatar_tokens (
   g_hash_table_destroy (ret);
 }
 
+static void
+salut_connection_avatars_fill_contact_attributes (GObject *obj,
+    const GArray *contacts, GHashTable *attributes_hash)
+{
+  guint i;
+  SalutConnection *self = SALUT_CONNECTION (obj);
+  TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+      gchar *token = NULL;
+
+      if (base->self_handle == handle)
+        {
+          token = g_strdup (priv->self->avatar_token);
+        }
+      else
+        {
+          SalutContact *contact = salut_contact_manager_get_contact (
+              priv->contact_manager, handle);
+          if (contact != NULL)
+            {
+              if (contact->avatar_token != NULL)
+                token = g_strdup (contact->avatar_token);
+              else
+                /* We always know the tokens, if it's unset then it's "" */
+                token = g_strdup ("");
+
+              g_object_unref (contact);
+            }
+        }
+
+      if (token != NULL)
+        {
+          GValue *val = tp_g_value_slice_new (G_TYPE_STRING);
+
+          g_value_take_string (val, token);
+
+          tp_contacts_mixin_set_contact_attribute (attributes_hash, handle,
+            TP_IFACE_CONNECTION_INTERFACE_AVATARS"/token", val);
+        }
+    }
+}
+
+
 static void
 _request_avatars_cb (SalutContact *contact, guint8 *avatar, gsize size,
     gpointer user_data)
@@ -1264,15 +1485,16 @@ salut_connection_request_avatars (
     const GArray *contacts,
     DBusGMethodInvocation *context)
 {
-  gint i;
+  guint i;
   GError *err = NULL;
   SalutConnection *self = SALUT_CONNECTION (iface);
   SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
   TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  TpHandleRepoIface *handle_repo;
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
 
-  TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (base,
+  handle_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
   if (!tp_handles_are_valid (handle_repo, contacts, FALSE, &err))
@@ -1353,10 +1575,11 @@ salut_connection_request_avatar (TpSvcConnectionInterfaceAvatars *iface,
   TpBaseConnection *base = TP_BASE_CONNECTION (self);
   SalutContact *contact;
   GError *err = NULL;
+  TpHandleRepoIface *handle_repo;
 
   TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
 
-  TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (base,
+  handle_repo = tp_base_connection_get_handles (base,
       TP_HANDLE_TYPE_CONTACT);
 
   if (!tp_handle_is_valid (handle_repo, handle, &err))
@@ -2372,6 +2595,7 @@ salut_connection_olpc_observe_invitation (SalutConnection *self,
 
   g_object_unref (muc);
   g_hash_table_destroy (properties);
+  g_object_unref (inviter);
 }
 
 static void
@@ -2428,7 +2652,7 @@ salut_connection_create_handle_repos (TpBaseConnection *self,
       list_handle_strings);
 }
 
-void
+static void
 _contact_manager_contact_change_cb (SalutContactManager *mgr,
     SalutContact *contact, int changes, gpointer data)
 {
@@ -2584,12 +2808,12 @@ uninvite_stanza_callback (SalutXmppConnectionManager *mgr,
 
 #endif
 
-static GPtrArray*
+static GPtrArray *
 salut_connection_create_channel_factories (TpBaseConnection *base)
 {
   SalutConnection *self = SALUT_CONNECTION (base);
   SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
-  GPtrArray *factories = g_ptr_array_sized_new (3);
+  GPtrArray *factories = g_ptr_array_sized_new (4);
 
   /* Create the contact manager */
   priv->contact_manager = salut_discovery_client_create_contact_manager (
@@ -2614,26 +2838,51 @@ salut_connection_create_channel_factories (TpBaseConnection *base)
       G_CALLBACK (_olpc_activity_manager_activity_modified_cb), self);
 #endif
 
+  return factories;
+}
+
+
+static GPtrArray *
+salut_connection_create_channel_managers (TpBaseConnection *base)
+{
+  SalutConnection *self = SALUT_CONNECTION (base);
+  SalutConnectionPrivate *priv = SALUT_CONNECTION_GET_PRIVATE (self);
+  GPtrArray *managers = g_ptr_array_sized_new (1);
+
+  /* FIXME: The second and third arguments depend on create_channel_factories
+   *        being called before this; should telepathy-glib guarantee that or
+   *        should we be defensive?
+   */
   priv->im_manager = salut_im_manager_new (self, priv->contact_manager,
       priv->xmpp_connection_manager);
 
+  priv->ft_manager = salut_ft_manager_new (self, priv->contact_manager,
+      priv->xmpp_connection_manager);
+
   priv->muc_manager = salut_discovery_client_create_muc_manager (
       priv->discovery_client, self, priv->xmpp_connection_manager);
 
-  /*
-  priv->tubes_manager = salut_tubes_manager_new (self, priv->contact_manager);
-  */
+  priv->roomlist_manager = salut_discovery_client_create_roomlist_manager (
+      priv->discovery_client, self, priv->xmpp_connection_manager);
 
-  g_ptr_array_add (factories, priv->contact_manager);
-  g_ptr_array_add (factories, priv->im_manager);
-  g_ptr_array_add (factories, priv->muc_manager);
-  /*
-  g_ptr_array_add (factories, priv->tubes_manager);
-  */
+#if 0
+  priv->tubes_manager = salut_tubes_manager_new (self, priv->contact_manager,
+      priv->xmpp_connection_manager);
+#endif
 
-  return factories;
+  g_ptr_array_add (managers, priv->im_manager);
+  g_ptr_array_add (managers, priv->contact_manager);
+  g_ptr_array_add (managers, priv->ft_manager);
+  g_ptr_array_add (managers, priv->muc_manager);
+  g_ptr_array_add (managers, priv->roomlist_manager);
+#if 0
+  g_ptr_array_add (managers, priv->tubes_manager);
+#endif
+
+  return managers;
 }
 
+
 static gchar *
 salut_connection_get_unique_connection_name (TpBaseConnection *base)
 {
