@@ -26,29 +26,32 @@
 #include "salut-connection.h"
 #include "salut-contact-channel.h"
 #include "salut-contact-manager.h"
-#include "signals-marshal.h"
+#include "salut-signals-marshal.h"
 #include "salut-contact.h"
 #include "salut-presence-enumtypes.h"
 
-#include <telepathy-glib/channel-factory-iface.h>
+#include <telepathy-glib/dbus.h>
 #include <telepathy-glib/interfaces.h>
 
 #define DEBUG_FLAG DEBUG_CONTACTS
 #include "debug.h"
 
-static void salut_contact_manager_factory_iface_init (gpointer *g_iface,
-    gpointer *iface_data);
+static void salut_contact_manager_manager_iface_init (gpointer g_iface,
+    gpointer iface_data);
 
 static SalutContactChannel *salut_contact_manager_get_channel
-    (SalutContactManager *mgr, TpHandle handle, gboolean *created);
+    (SalutContactManager *mgr, TpHandle handle, gpointer request_token,
+    gboolean *created);
+
+static void salut_contact_manager_close_all (SalutContactManager *mgr);
 
 static void
 _contact_finalized_cb (gpointer data, GObject *old_object);
 
 G_DEFINE_TYPE_WITH_CODE(SalutContactManager, salut_contact_manager,
     G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
-      salut_contact_manager_factory_iface_init));
+    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_MANAGER,
+      salut_contact_manager_manager_iface_init));
 
 enum
 {
@@ -71,6 +74,7 @@ typedef struct _SalutContactManagerPrivate SalutContactManagerPrivate;
 struct _SalutContactManagerPrivate
 {
   GHashTable *channels;
+  gulong status_changed_id;
   gboolean dispose_has_run;
 };
 
@@ -125,6 +129,8 @@ salut_contact_manager_init (SalutContactManager *obj)
   obj->contacts = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
+static GObject *salut_contact_manager_constructor (GType type,
+    guint n_props, GObjectConstructParam *props);
 static void salut_contact_manager_dispose (GObject *object);
 static void salut_contact_manager_finalize (GObject *object);
 
@@ -139,6 +145,7 @@ salut_contact_manager_class_init (SalutContactManagerClass *salut_contact_manage
   object_class->get_property = salut_contact_manager_get_property;
   object_class->set_property = salut_contact_manager_set_property;
 
+  object_class->constructor = salut_contact_manager_constructor;
   object_class->dispose = salut_contact_manager_dispose;
   object_class->finalize = salut_contact_manager_finalize;
 
@@ -148,10 +155,7 @@ salut_contact_manager_class_init (SalutContactManagerClass *salut_contact_manage
       "The Salut Connection associated with this contact manager",
       SALUT_TYPE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION,
       param_spec);
 
@@ -166,6 +170,39 @@ salut_contact_manager_class_init (SalutContactManagerClass *salut_contact_manage
       G_TYPE_INT);
 }
 
+static void
+connection_status_changed_cb (SalutConnection *conn,
+                              guint status,
+                              guint reason,
+                              SalutContactManager *self)
+{
+  if (status == TP_CONNECTION_STATUS_DISCONNECTED)
+    {
+      salut_contact_manager_close_all (self);
+    }
+}
+
+static GObject *
+salut_contact_manager_constructor (GType type,
+    guint n_props,
+    GObjectConstructParam *props)
+{
+  GObject *obj;
+  SalutContactManager *self;
+  SalutContactManagerPrivate *priv;
+
+  obj = G_OBJECT_CLASS (salut_contact_manager_parent_class)->
+           constructor (type, n_props, props);
+
+  self = SALUT_CONTACT_MANAGER (obj);
+  priv = SALUT_CONTACT_MANAGER_GET_PRIVATE (self);
+
+  priv->status_changed_id = g_signal_connect (self->connection,
+      "status-changed", (GCallback) connection_status_changed_cb, self);
+
+  return obj;
+}
+
 static gboolean
 dispose_contact (gpointer key, gpointer value, gpointer object)
 {
@@ -194,7 +231,7 @@ salut_contact_manager_dispose (GObject *object)
   priv->dispose_has_run = TRUE;
 
   /* release any references held by the object here */
-  tp_channel_factory_iface_close_all (TP_CHANNEL_FACTORY_IFACE (object));
+  salut_contact_manager_close_all (self);
 
   if (G_OBJECT_CLASS (salut_contact_manager_parent_class)->dispose)
     G_OBJECT_CLASS (salut_contact_manager_parent_class)->dispose (object);
@@ -220,7 +257,7 @@ change_all_groups (SalutContactManager *mgr, TpIntSet *add, TpIntSet *rem)
 
   for (i = LIST_HANDLE_FIRST; i <= LIST_HANDLE_LAST; i++)
     {
-      c = salut_contact_manager_get_channel (mgr, i, NULL);
+      c = salut_contact_manager_get_channel (mgr, i, NULL, NULL);
       tp_group_mixin_change_members (G_OBJECT(c),
                                      "", add, rem,
                                      empty, empty, 0, 0);
@@ -325,9 +362,8 @@ salut_contact_manager_ensure_contact (SalutContactManager *self,
 }
 
 static void
-salut_contact_manager_factory_iface_close_all (TpChannelFactoryIface *iface)
+salut_contact_manager_close_all (SalutContactManager *mgr)
 {
-  SalutContactManager *mgr = SALUT_CONTACT_MANAGER (iface);
   SalutContactManagerPrivate *priv =
     SALUT_CONTACT_MANAGER_GET_PRIVATE (mgr);
 
@@ -345,101 +381,200 @@ salut_contact_manager_factory_iface_close_all (TpChannelFactoryIface *iface)
       g_hash_table_destroy (mgr->contacts);
       mgr->contacts = NULL;
     }
+
+  if (priv->status_changed_id != 0)
+    {
+      g_signal_handler_disconnect (mgr->connection, priv->status_changed_id);
+      priv->status_changed_id = 0;
+    }
 }
 
-static void
-salut_contact_manager_factory_iface_connecting (TpChannelFactoryIface *iface)
-{
-}
-
-static void
-salut_contact_manager_factory_iface_connected (TpChannelFactoryIface *iface)
-{
-}
-
-static void
-salut_contact_manager_factory_iface_disconnected (TpChannelFactoryIface *iface)
-{
-}
-
-struct foreach_data {
-  TpChannelFunc func;
+/* TpChannelManager implementation */
+struct foreach_channel_data {
+  TpExportableChannelFunc func;
   gpointer data;
 };
 
 static void
-salut_contact_manager_iface_foreach_one (gpointer key, gpointer value,
+salut_contact_manager_foreach_one (gpointer key,
+    gpointer value,
     gpointer data)
 {
-  TpChannelIface *chan = TP_CHANNEL_IFACE(value);
-  struct foreach_data *f = (struct foreach_data *) data;
+  TpExportableChannel *chan = TP_EXPORTABLE_CHANNEL (value);
+  struct foreach_channel_data *f = data;
 
   f->func (chan, f->data);
 }
 
 static void
-salut_contact_manager_factory_iface_foreach (TpChannelFactoryIface *iface,
-    TpChannelFunc func, gpointer data)
+salut_contact_manager_foreach_channel (TpChannelManager *iface,
+    TpExportableChannelFunc func,
+    gpointer data)
 {
   SalutContactManager *mgr = SALUT_CONTACT_MANAGER (iface);
   SalutContactManagerPrivate *priv = SALUT_CONTACT_MANAGER_GET_PRIVATE (mgr);
-  struct foreach_data f;
+  struct foreach_channel_data f;
   f.func = func;
   f.data = data;
 
   g_hash_table_foreach (priv->channels,
-                        salut_contact_manager_iface_foreach_one, &f);
+                        salut_contact_manager_foreach_one, &f);
 }
 
-static TpChannelFactoryRequestStatus
-salut_contact_manager_factory_iface_request (TpChannelFactoryIface *iface,
-    const gchar *chan_type, TpHandleType handle_type,
-    guint handle, gpointer request,
-    TpChannelIface **ret, GError **error)
+static const gchar * const list_channel_fixed_properties[] = {
+    TP_IFACE_CHANNEL ".ChannelType",
+    TP_IFACE_CHANNEL ".TargetHandleType",
+    NULL
+};
+
+static const gchar * const list_channel_allowed_properties[] = {
+    TP_IFACE_CHANNEL ".TargetHandle",
+    TP_IFACE_CHANNEL ".TargetID",
+    NULL
+};
+
+static void
+salut_contact_manager_foreach_channel_class (TpChannelManager *manager,
+    TpChannelManagerChannelClassFunc func,
+    gpointer user_data)
+{
+  GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
+      NULL, (GDestroyNotify) tp_g_value_slice_free);
+  GValue *value, *handle_type_value;
+
+  value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_static_string (value, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST);
+  g_hash_table_insert (table, TP_IFACE_CHANNEL ".ChannelType", value);
+
+  handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (handle_type_value, TP_HANDLE_TYPE_LIST);
+  g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
+      handle_type_value);
+
+  func (manager, table, list_channel_allowed_properties, user_data);
+
+  g_hash_table_destroy (table);
+}
+
+static gboolean
+salut_contact_manager_request (SalutContactManager *self,
+    gpointer request_token,
+    GHashTable *request_properties,
+    gboolean require_new)
 {
-  SalutContactManager *mgr = SALUT_CONTACT_MANAGER(iface);
-  SalutContactChannel *chan;
-  gboolean created;
   TpHandleRepoIface *handle_repo = tp_base_connection_get_handles (
-      TP_BASE_CONNECTION (mgr->connection), TP_HANDLE_TYPE_LIST);
+      TP_BASE_CONNECTION (self->connection), TP_HANDLE_TYPE_LIST);
+  TpHandleType handle_type;
+  TpHandle handle;
+  SalutContactChannel *channel;
+  gboolean created;
+  GError *error = NULL;
 
-  /* We only support contact list channels */
-  if (tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST)) {
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_IMPLEMENTED;
-  }
+  if (tp_strdiff (tp_asv_get_string (request_properties,
+          TP_IFACE_CHANNEL ".ChannelType"),
+        TP_IFACE_CHANNEL_TYPE_CONTACT_LIST))
+    return FALSE;
 
-  /* And thus only support list handles */
-  if (handle_type != TP_HANDLE_TYPE_LIST) {
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
-  }
+  handle_type = tp_asv_get_uint32 (request_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", NULL);
 
-  /* Most be a valid list handle */
-  if (!tp_handle_is_valid (handle_repo, handle, NULL)) {
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
-  }
+  if (handle_type != TP_HANDLE_TYPE_LIST)
+    return FALSE;
 
-  chan = salut_contact_manager_get_channel (mgr, handle, &created);
-  *ret = TP_CHANNEL_IFACE (chan);
-  return created ? TP_CHANNEL_FACTORY_REQUEST_STATUS_CREATED
-                 : TP_CHANNEL_FACTORY_REQUEST_STATUS_EXISTING;
+  handle = tp_asv_get_uint32 (request_properties,
+      TP_IFACE_CHANNEL ".TargetHandle", NULL);
+  g_assert (tp_handle_is_valid (handle_repo, handle, NULL));
+
+  /* Check if there are any other properties that we don't understand */
+  if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+          list_channel_fixed_properties, list_channel_allowed_properties,
+          &error))
+    {
+      goto error;
+    }
+
+  channel = salut_contact_manager_get_channel (self, handle, request_token,
+      &created);
+
+  if (created)
+    {
+      /* Do nothing; salut_contact_manager_new_channel emits the new-channel
+       * signal
+       */
+    }
+  else
+    {
+      if (require_new)
+        {
+          g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+            "Contact list channel #%u already exists", handle);
+          goto error;
+        }
+      else
+        {
+          tp_channel_manager_emit_request_already_satisfied (self,
+              request_token, TP_EXPORTABLE_CHANNEL (channel));
+        }
+    }
+  return TRUE;
+
+error:
+  tp_channel_manager_emit_request_failed (self, request_token,
+      error->domain, error->code, error->message);
+  g_error_free (error);
+  return FALSE;
 }
 
-static void salut_contact_manager_factory_iface_init (gpointer *g_iface,
-    gpointer *iface_data)
+static gboolean
+salut_contact_manager_request_channel (TpChannelManager *manager,
+    gpointer request_token,
+    GHashTable *request_properties)
 {
-  TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *)g_iface;
+  SalutContactManager *self = SALUT_CONTACT_MANAGER (manager);
 
-  klass->close_all = salut_contact_manager_factory_iface_close_all;
-  klass->connecting = salut_contact_manager_factory_iface_connecting;
-  klass->connected = salut_contact_manager_factory_iface_connected;
-  klass->disconnected = salut_contact_manager_factory_iface_disconnected;
-  klass->foreach = salut_contact_manager_factory_iface_foreach;
-  klass->request = salut_contact_manager_factory_iface_request;
+  return salut_contact_manager_request (self, request_token,
+      request_properties, FALSE);
+}
+
+static gboolean
+salut_contact_manager_create_channel (TpChannelManager *manager,
+    gpointer request_token,
+    GHashTable *request_properties)
+{
+  SalutContactManager *self = SALUT_CONTACT_MANAGER (manager);
+
+  return salut_contact_manager_request (self, request_token,
+      request_properties, TRUE);
+}
+
+static gboolean
+salut_contact_manager_ensure_channel (TpChannelManager *manager,
+    gpointer request_token,
+    GHashTable *request_properties)
+{
+  SalutContactManager *self = SALUT_CONTACT_MANAGER (manager);
+
+  return salut_contact_manager_request (self, request_token,
+      request_properties, FALSE);
+}
+
+static void
+salut_contact_manager_manager_iface_init (gpointer g_iface,
+    gpointer iface_data)
+{
+  TpChannelManagerIface *iface = g_iface;
+
+  iface->foreach_channel = salut_contact_manager_foreach_channel;
+  iface->foreach_channel_class = salut_contact_manager_foreach_channel_class;
+  iface->request_channel = salut_contact_manager_request_channel;
+  iface->create_channel = salut_contact_manager_create_channel;
+  iface->ensure_channel = salut_contact_manager_ensure_channel;
 }
 
 /* private functions */
 static SalutContactChannel *
 salut_contact_manager_new_channel (SalutContactManager *mgr,
+    gpointer request_token,
     TpHandle handle)
 {
   SalutContactManagerPrivate *priv = SALUT_CONTACT_MANAGER_GET_PRIVATE (mgr);
@@ -449,6 +584,7 @@ salut_contact_manager_new_channel (SalutContactManager *mgr,
   SalutContactChannel *chan;
   const gchar *name;
   gchar *path;
+  GSList *requests = NULL;
 
   g_assert (g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle))
              == NULL);
@@ -461,18 +597,25 @@ salut_contact_manager_new_channel (SalutContactManager *mgr,
       "connection", mgr->connection,
       "object-path", path,
       "handle", handle,
+      "requested", (request_token != NULL),
       NULL);
   g_free (path);
   g_hash_table_insert (priv->channels, GUINT_TO_POINTER (handle), chan);
-  tp_channel_factory_iface_emit_new_channel (mgr, TP_CHANNEL_IFACE (chan),
-      NULL);
+
+  if (request_token != NULL)
+    requests = g_slist_prepend (requests, request_token);
+
+  tp_channel_manager_emit_new_channel (mgr, TP_EXPORTABLE_CHANNEL (chan),
+      requests);
+
+  g_slist_free (requests);
 
   return chan;
 }
 
 static SalutContactChannel *
 salut_contact_manager_get_channel (SalutContactManager *mgr,
-    TpHandle handle, gboolean *created)
+    TpHandle handle, gpointer request_token, gboolean *created)
 {
   SalutContactManagerPrivate *priv = SALUT_CONTACT_MANAGER_GET_PRIVATE (mgr);
   SalutContactChannel *chan;
@@ -484,7 +627,7 @@ salut_contact_manager_get_channel (SalutContactManager *mgr,
     }
   if (chan == NULL)
     {
-      chan = salut_contact_manager_new_channel (mgr, handle);
+      chan = salut_contact_manager_new_channel (mgr, request_token, handle);
     }
 
   return chan;
@@ -520,13 +663,15 @@ salut_contact_manager_get_contact (SalutContactManager *mgr, TpHandle handle)
 }
 
 static void
-_find_by_address (gpointer key, gpointer value, gpointer user_data) {
-  struct sockaddr_storage *address =
-    (struct sockaddr_storage *)((gpointer *)user_data)[0];
-  GList **list = (GList **)((gpointer *)user_data)[1];
+_find_by_address (gpointer key, gpointer value, gpointer user_data)
+{
+  struct sockaddr *address =
+    (struct sockaddr *)((gpointer *) user_data)[0];
+  GList **list = (GList **)((gpointer *) user_data)[1];
+  guint size = GPOINTER_TO_UINT (((gpointer *) user_data)[2]);
   SalutContact *contact = SALUT_CONTACT (value);
 
-  if (salut_contact_has_address (contact, address)) {
+  if (salut_contact_has_address (contact, address, size)) {
     g_object_ref (contact);
     *list = g_list_append (*list, contact);
   }
@@ -535,13 +680,14 @@ _find_by_address (gpointer key, gpointer value, gpointer user_data) {
 /* FIXME function name is just too long */
 GList *
 salut_contact_manager_find_contacts_by_address (SalutContactManager *mgr,
-    struct sockaddr_storage *address)
+    struct sockaddr *address, guint size)
 {
   GList *list = NULL;
-  gpointer data[2];
+  gpointer data[3];
 
   data[0] = address;
   data[1] = &list;
+  data[2] = GUINT_TO_POINTER (size);
   g_hash_table_foreach (mgr->contacts, _find_by_address, data);
   return list;
 }
