@@ -22,7 +22,7 @@
 #include <string.h>
 
 #include "salut-contact.h"
-#include "signals-marshal.h"
+#include "salut-signals-marshal.h"
 #include "salut-presence.h"
 #include "salut-presence-enumtypes.h"
 
@@ -225,10 +225,7 @@ salut_contact_class_init (SalutContactClass *salut_contact_class)
       "The Salut Connection associated with this contact",
       SALUT_TYPE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION,
       param_spec);
 
@@ -238,10 +235,7 @@ salut_contact_class_init (SalutContactClass *salut_contact_class)
       "The name of this contact",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_NAME,
       param_spec);
 }
@@ -387,9 +381,10 @@ salut_contact_get_addresses (SalutContact *self)
 
 gboolean
 salut_contact_has_address (SalutContact *self,
-                           struct sockaddr_storage *address)
+                           struct sockaddr *address,
+                           guint size)
 {
-  return SALUT_CONTACT_GET_CLASS (self)->has_address (self, address);
+  return SALUT_CONTACT_GET_CLASS (self)->has_address (self, address, size);
 }
 
 const gchar *
@@ -416,7 +411,7 @@ salut_contact_avatar_request_flush (SalutContact *contact,
   priv->avatar_requests = NULL;
 
   for (list = liststart; list != NULL; list = g_list_next (list)) {
-    request = (AvatarRequest *)list->data;
+    request = (AvatarRequest *) list->data;
     request->callback (contact, data, size, request->user_data);
     g_slice_free (AvatarRequest, request);
   }
