@@ -37,6 +37,7 @@
 
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/svc-generic.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/util.h>
@@ -54,13 +55,21 @@ static void channel_iface_init (gpointer g_iface, gpointer iface_data);
 static void text_iface_init (gpointer g_iface, gpointer iface_data);
 
 G_DEFINE_TYPE_WITH_CODE(SalutMucChannel, salut_muc_channel, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
+      tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL, channel_iface_init);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_IFACE, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_GROUP,
         tp_group_mixin_iface_init);
     G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CHANNEL_TYPE_TEXT, text_iface_init);
 )
 
+static const char *salut_muc_channel_interfaces[] = {
+  TP_IFACE_CHANNEL_INTERFACE_GROUP,
+  NULL
+};
+
 /* signal enum */
 enum
 {
@@ -83,6 +92,13 @@ enum
   PROP_NAME,
   PROP_CREATOR,
   PROP_XMPP_CONNECTION_MANAGER,
+  PROP_INTERFACES,
+  PROP_TARGET_ID,
+  PROP_REQUESTED,
+  PROP_INITIATOR_HANDLE,
+  PROP_INITIATOR_ID,
+  PROP_CHANNEL_PROPERTIES,
+  PROP_CHANNEL_DESTROYED,
   LAST_PROPERTY
 };
 
@@ -94,7 +110,6 @@ struct _SalutMucChannelPrivate
   gboolean dispose_has_run;
   gchar *object_path;
   TpHandle handle;
-  SalutConnection *connection;
   SalutSelf *self;
   SalutXmppConnectionManager *xmpp_connection_manager;
   GibberMucConnection *muc_connection;
@@ -105,6 +120,9 @@ struct _SalutMucChannelPrivate
   /* (gchar *) -> (SalutContact *) */
   GHashTable *senders;
   SalutMucManager *muc_manager;
+  TpHandle initiator;
+  gboolean requested;
+  gboolean closed;
 };
 
 #define SALUT_MUC_CHANNEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), SALUT_TYPE_MUC_CHANNEL, SalutMucChannelPrivate))
@@ -132,6 +150,7 @@ salut_muc_channel_get_property (GObject    *object,
 {
   SalutMucChannel *chan = SALUT_MUC_CHANNEL (object);
   SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (chan);
+  TpBaseConnection *base_conn = (TpBaseConnection *) chan->connection;
 
   switch (property_id) {
     case PROP_OBJECT_PATH:
@@ -161,6 +180,50 @@ salut_muc_channel_get_property (GObject    *object,
     case PROP_XMPP_CONNECTION_MANAGER:
       g_value_set_object (value, priv->xmpp_connection_manager);
       break;
+    case PROP_INTERFACES:
+      g_value_set_static_boxed (value, salut_muc_channel_interfaces);
+      break;
+    case PROP_TARGET_ID:
+      {
+        TpHandleRepoIface *repo = tp_base_connection_get_handles (base_conn,
+            TP_HANDLE_TYPE_ROOM);
+
+        g_value_set_string (value, tp_handle_inspect (repo, priv->handle));
+      }
+      break;
+    case PROP_INITIATOR_HANDLE:
+      g_value_set_uint (value, priv->initiator);
+      break;
+    case PROP_INITIATOR_ID:
+      {
+        TpHandleRepoIface *repo = tp_base_connection_get_handles (base_conn,
+            TP_HANDLE_TYPE_CONTACT);
+
+        g_value_set_string (value, tp_handle_inspect (repo, priv->initiator));
+      }
+      break;
+    case PROP_REQUESTED:
+      g_value_set_boolean (value, priv->requested);
+      break;
+      case PROP_CHANNEL_DESTROYED:
+        /* TODO: this should be FALSE if there are still pending messages, so
+         *       the channel manager can respawn the channel.
+         */
+        g_value_set_boolean (value, TRUE);
+        break;
+      case PROP_CHANNEL_PROPERTIES:
+        g_value_take_boxed (value,
+            tp_dbus_properties_mixin_make_properties_hash (object,
+                TP_IFACE_CHANNEL, "TargetHandle",
+                TP_IFACE_CHANNEL, "TargetHandleType",
+                TP_IFACE_CHANNEL, "ChannelType",
+                TP_IFACE_CHANNEL, "TargetID",
+                TP_IFACE_CHANNEL, "InitiatorHandle",
+                TP_IFACE_CHANNEL, "InitiatorID",
+                TP_IFACE_CHANNEL, "Requested",
+                TP_IFACE_CHANNEL, "Interfaces",
+                NULL));
+        break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -208,9 +271,15 @@ salut_muc_channel_set_property (GObject     *object,
     case PROP_CREATOR:
       priv->creator = g_value_get_boolean (value);
       break;
-   case PROP_XMPP_CONNECTION_MANAGER:
+    case PROP_INITIATOR_HANDLE:
+      priv->initiator = g_value_get_uint (value);
+      break;
+    case PROP_XMPP_CONNECTION_MANAGER:
       priv->xmpp_connection_manager = g_value_get_object (value);
       break;
+    case PROP_REQUESTED:
+      priv->requested = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -452,6 +521,7 @@ pending_connection_for_invite_data_free (
     struct pending_connection_for_invite_data *data)
 {
   g_object_unref (data->invite);
+  g_object_unref (data->contact);
   g_slice_free (struct pending_connection_for_invite_data, data);
 }
 
@@ -559,6 +629,7 @@ salut_muc_channel_send_invitation (SalutMucChannel *self,
       SALUT_XMPP_CONNECTION_MANAGER_REQUEST_CONNECTION_RESULT_FAILURE)
     {
       DEBUG ("request connection failed");
+      g_object_unref (contact);
       return FALSE;
     }
 
@@ -570,6 +641,7 @@ salut_muc_channel_send_invitation (SalutMucChannel *self,
       DEBUG ("got an existing connection. Send the invite");
       result = gibber_xmpp_connection_send (connection, stanza, error);
       g_object_unref (stanza);
+      g_object_unref (contact);
       return TRUE;
     }
 
@@ -596,14 +668,16 @@ salut_muc_channel_add_member (GObject *iface,
                               GError **error)
 {
   SalutMucChannel *self = SALUT_MUC_CHANNEL(iface);
+#ifdef ENABLE_DEBUG
   SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
+#endif
   TpBaseConnection *base_connection = TP_BASE_CONNECTION (self->connection);
   TpIntSet *empty, *remote_pending;
 
   if (handle == base_connection->self_handle)
     {
       /* adding yourself, let's join the muc */
-      TpIntSet *empty;
+      TpIntSet *empty_;
       TpIntSet *add;
       gboolean ret = TRUE;
 
@@ -614,7 +688,7 @@ salut_muc_channel_add_member (GObject *iface,
           return TRUE;
         }
 
-      empty = tp_intset_new ();
+      empty_ = tp_intset_new ();
       add = tp_intset_new ();
       tp_intset_add (add, handle);
       /* Add to members */
@@ -624,7 +698,8 @@ salut_muc_channel_add_member (GObject *iface,
           /* We are considered as remote-pending while the muc connection
            * is not connected */
           tp_group_mixin_change_members (G_OBJECT (self),
-              message, empty, empty, empty, add, base_connection->self_handle,
+              message, empty_, empty_, empty_, add,
+              base_connection->self_handle,
               TP_CHANNEL_GROUP_CHANGE_REASON_NONE);
         }
       else
@@ -634,7 +709,7 @@ salut_muc_channel_add_member (GObject *iface,
           ret = FALSE;
         }
 
-      tp_intset_destroy (empty);
+      tp_intset_destroy (empty_);
       tp_intset_destroy (add);
       return ret;
     }
@@ -663,8 +738,28 @@ static void
 salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
   GObjectClass *object_class = G_OBJECT_CLASS (salut_muc_channel_class);
   GParamSpec *param_spec;
+  static TpDBusPropertiesMixinPropImpl channel_props[] = {
+      { "TargetHandleType", "handle-type", NULL },
+      { "TargetHandle", "handle", NULL },
+      { "TargetID", "target-id", NULL },
+      { "ChannelType", "channel-type", NULL },
+      { "Interfaces", "interfaces", NULL },
+      { "Requested", "requested", NULL },
+      { "InitiatorHandle", "initiator-handle", NULL },
+      { "InitiatorID", "initiator-id", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+      { TP_IFACE_CHANNEL,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        channel_props,
+      },
+      { NULL }
+  };
 
-  g_type_class_add_private (salut_muc_channel_class, sizeof (SalutMucChannelPrivate));
+  g_type_class_add_private (salut_muc_channel_class,
+      sizeof (SalutMucChannelPrivate));
 
   object_class->dispose = salut_muc_channel_dispose;
   object_class->finalize = salut_muc_channel_finalize;
@@ -680,6 +775,38 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
   g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
                                     "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
+  g_object_class_override_property (object_class, PROP_CHANNEL_DESTROYED,
+      "channel-destroyed");
+  g_object_class_override_property (object_class, PROP_CHANNEL_PROPERTIES,
+      "channel-properties");
+
+
+  param_spec = g_param_spec_string ("target-id", "Target JID",
+      "The string obtained by inspecting this channel's handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
+
+  param_spec = g_param_spec_boolean ("requested", "Requested?",
+      "True if this channel was requested by the local user",
+      FALSE,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_REQUESTED, param_spec);
+
+  param_spec = g_param_spec_uint ("initiator-handle", "Initiator's handle",
+      "The contact which invited us to the MUC, or ourselves if we joined of "
+      "our own accord",
+      0, G_MAXUINT32, 0,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_HANDLE,
+      param_spec);
+
+  param_spec = g_param_spec_string ("initiator-id", "Initiator JID",
+      "The string obtained by inspecting this channel's initiator-handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_ID,
+      param_spec);
 
   param_spec = g_param_spec_string ("name",
                                     "Name of the muc group",
@@ -687,8 +814,7 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
                                     NULL,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+                                    G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class,
                                    PROP_NAME, param_spec);
 
@@ -698,8 +824,7 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
                                     GIBBER_TYPE_MUC_CONNECTION,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+                                    G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class,
                                    PROP_MUC_CONNECTION, param_spec);
   param_spec = g_param_spec_object ("connection",
@@ -709,8 +834,7 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
                                     SALUT_TYPE_CONNECTION,
                                     G_PARAM_CONSTRUCT_ONLY |
                                     G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
+                                    G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class,
                                    PROP_CONNECTION, param_spec);
 
@@ -721,8 +845,7 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
       SALUT_TYPE_XMPP_CONNECTION_MANAGER,
       G_PARAM_CONSTRUCT_ONLY |
       G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_XMPP_CONNECTION_MANAGER,
       param_spec);
 
@@ -733,11 +856,17 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
       FALSE,
       G_PARAM_CONSTRUCT_ONLY |
       G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class,
       PROP_CREATOR, param_spec);
 
+  param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
+      "Additional Channel.Interface.* interfaces",
+      G_TYPE_STRV,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
+
   signals[READY] = g_signal_new (
         "ready",
         G_OBJECT_CLASS_TYPE (salut_muc_channel_class),
@@ -756,12 +885,17 @@ salut_muc_channel_class_init (SalutMucChannelClass *salut_muc_channel_class) {
         g_cclosure_marshal_VOID__POINTER,
         G_TYPE_NONE, 1, G_TYPE_POINTER);
 
+  salut_muc_channel_class->dbus_props_class.interfaces = prop_interfaces;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (SalutMucChannelClass, dbus_props_class));
+
   tp_text_mixin_class_init (object_class,
       G_STRUCT_OFFSET(SalutMucChannelClass, text_class));
 
   tp_group_mixin_class_init (object_class,
       G_STRUCT_OFFSET(SalutMucChannelClass, group_class),
       salut_muc_channel_add_member, NULL);
+  tp_group_mixin_init_dbus_properties (object_class);
 }
 
 void
@@ -796,6 +930,12 @@ salut_muc_channel_dispose (GObject *object)
       priv->senders = NULL;
     }
 
+  if (!priv->closed)
+    {
+      priv->closed = TRUE;
+      tp_svc_channel_emit_closed (self);
+    }
+
   /* release any references held by the object here */
   if (G_OBJECT_CLASS (salut_muc_channel_parent_class)->dispose)
     G_OBJECT_CLASS (salut_muc_channel_parent_class)->dispose (object);
@@ -821,12 +961,14 @@ gboolean
 salut_muc_channel_invited (SalutMucChannel *self, TpHandle inviter,
                           const gchar *stanza, GError **error)
 {
-  SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
   TpBaseConnection *base_connection = TP_BASE_CONNECTION (self->connection);
+#ifdef ENABLE_DEBUG
+  SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
   TpHandleRepoIface *contact_repo =
       tp_base_connection_get_handles (base_connection, TP_HANDLE_TYPE_CONTACT);
   TpHandleRepoIface *room_repo =
       tp_base_connection_get_handles (base_connection, TP_HANDLE_TYPE_ROOM);
+#endif
   gboolean ret = TRUE;
 
   /* Got invited to this muc channel */
@@ -1016,10 +1158,10 @@ salut_muc_channel_received_stanza (GibberMucConnection *conn,
       SalutTubesChannel *tubes_chan;
 
       tubes_chan = salut_muc_manager_ensure_tubes_channel (priv->muc_manager,
-          priv->handle);
+          priv->handle, from_handle);
       g_assert (tubes_chan != NULL);
 
-      tubes_message_received (tubes_chan, sender, stanza);
+      salut_tubes_channel_muc_message_received (tubes_chan, sender, stanza);
 
       g_object_unref (tubes_chan);
     }
@@ -1109,12 +1251,10 @@ salut_muc_channel_disconnected (GibberTransport *transport, gpointer user_data)
       g_signal_emit (self, signals[JOIN_ERROR], 0, &error);
     }
 
-  tp_svc_channel_emit_closed (self);
-}
+  if (priv->closed)
+    return;
 
-void
-salut_muc_channel_emit_closed (SalutMucChannel *self)
-{
+  priv->closed = TRUE;
   tp_svc_channel_emit_closed (self);
 }
 
@@ -1135,9 +1275,8 @@ static void
 salut_muc_channel_get_interfaces (TpSvcChannel *iface,
     DBusGMethodInvocation *context)
 {
-  const char *interfaces[] = { TP_IFACE_CHANNEL_INTERFACE_GROUP,  NULL };
-
-  tp_svc_channel_return_from_get_interfaces (context, interfaces);
+  tp_svc_channel_return_from_get_interfaces (context,
+    salut_muc_channel_interfaces);
 }
 
 
@@ -1201,7 +1340,26 @@ salut_muc_channel_close (TpSvcChannel *iface, DBusGMethodInvocation *context)
   SalutMucChannel *self = SALUT_MUC_CHANNEL (iface);
   SalutMucChannelPrivate *priv = SALUT_MUC_CHANNEL_GET_PRIVATE (self);
 
-  gibber_muc_connection_disconnect (priv->muc_connection);
+  if (priv->closed)
+    {
+      GError already = { TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+          "Channel already closed"};
+      DEBUG ("channel already closed");
+
+      dbus_g_method_return_error (context, &already);
+      return;
+    }
+
+  if (priv->connected)
+    {
+      /* priv->closed will be set in salut_muc_channel_disconnected */
+      gibber_muc_connection_disconnect (priv->muc_connection);
+    }
+  else
+    {
+      priv->closed = TRUE;
+      tp_svc_channel_emit_closed (self);
+    }
 
   tp_svc_channel_return_from_close (context);
 }
@@ -1209,7 +1367,7 @@ salut_muc_channel_close (TpSvcChannel *iface, DBusGMethodInvocation *context)
 static void
 channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
+  TpSvcChannelClass *klass = (TpSvcChannelClass *) g_iface;
 
 #define IMPLEMENT(x) tp_svc_channel_implement_##x (\
     klass, salut_muc_channel_##x)
@@ -1274,7 +1432,7 @@ salut_muc_channel_send (TpSvcChannelTypeText *channel,
 static void
 text_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *)g_iface;
+  TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *) g_iface;
 
   tp_text_mixin_iface_init (g_iface, iface_data);
 #define IMPLEMENT(x) tp_svc_channel_type_text_implement_##x (\
