@@ -326,10 +326,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The Salut Connection associated with this self object",
       SALUT_TYPE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION,
       param_spec);
 
@@ -339,10 +336,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The nickname of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_NICKNAME,
       param_spec);
 
@@ -352,10 +346,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The first name of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_FIRST_NAME,
       param_spec);
 
@@ -365,10 +356,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The last name of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_LAST_NAME,
       param_spec);
 
@@ -378,10 +366,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The jabber ID of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_JID,
       param_spec);
 
@@ -391,10 +376,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The email of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_EMAIL,
       param_spec);
 
@@ -404,10 +386,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The name used to publish the presence service",
       g_get_user_name (),
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_PUBLISHED_NAME,
       param_spec);
 
@@ -417,10 +396,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "the OLPC key",
       "A pointer to a GArray containing the OLPC key",
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_OLPC_KEY,
       param_spec);
 
@@ -430,10 +406,7 @@ salut_self_class_init (SalutSelfClass *salut_self_class)
       "The OLPC color of the self user",
       NULL,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NAME |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_OLPC_COLOR,
       param_spec);
 #endif
@@ -545,7 +518,7 @@ salut_self_set_presence (SalutSelf *self, SalutPresenceId status,
     const gchar *message, GError **error)
 {
 
-  g_assert (status >= 0 && status < SALUT_PRESENCE_NR_PRESENCES);
+  g_assert (status < SALUT_PRESENCE_NR_PRESENCES);
 
   self->status = status;
   g_free (self->status_message);
@@ -578,7 +551,9 @@ salut_self_set_alias (SalutSelf *self, const gchar *alias, GError **error)
   ret = SALUT_SELF_GET_CLASS (self)->set_alias (self, &err);
   if (!ret)
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, err->message);
+      if (error != NULL)
+        *error = g_error_new_literal (TP_ERRORS, TP_ERROR_NETWORK_ERROR,
+            err->message);
       g_error_free (err);
     }
   return ret;
@@ -619,7 +594,9 @@ salut_self_set_avatar (SalutSelf *self, guint8 *data,
   if (!ret)
     {
       salut_self_remove_avatar (self);
-      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, err->message);
+      if (error != NULL)
+        *error = g_error_new_literal (TP_ERRORS, TP_ERROR_NETWORK_ERROR,
+            err->message);
       g_error_free (err);
     }
 
@@ -709,18 +686,18 @@ _set_olpc_activities_delete (gpointer key, gpointer value, gpointer user_data)
 {
   SalutOlpcActivity *activity = (SalutOlpcActivity *) value;
   struct _set_olpc_activities_ctx *data = user_data;
-  gboolean remove;
+  gboolean remove_activity;
 
   /* delete the activity service if it's not in data->room_to_act_id */
-  remove = (g_hash_table_lookup (data->room_to_act_id, key) == NULL);
+  remove_activity = (g_hash_table_lookup (data->room_to_act_id, key) == NULL);
 
-  if (remove)
+  if (remove_activity)
     {
       salut_olpc_activity_left (activity);
       salut_olpc_activity_revoke_invitations (activity);
     }
 
-  return remove;
+  return remove_activity;
 }
 
 gboolean
@@ -795,7 +772,8 @@ salut_self_set_olpc_current_activity (SalutSelf *self,
   if (!SALUT_SELF_GET_CLASS (self)->update_current_activity (self, room_name,
         &err))
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, err->message);
+      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "%s",
+          err->message);
       g_error_free (err);
       return FALSE;
     }
@@ -873,7 +851,8 @@ salut_self_set_olpc_properties (SalutSelf *self,
   if (!SALUT_SELF_GET_CLASS (self)->set_olpc_properties (self, key, color, jid,
         &err))
     {
-      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, err->message);
+      g_set_error (error, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "%s",
+          err->message);
       g_error_free (err);
       return FALSE;
     }
