@@ -24,8 +24,10 @@
 #include <gibber/gibber-xmpp-stanza.h>
 #include <gibber/gibber-xmpp-connection.h>
 #include <gibber/gibber-bytestream-iface.h>
+#include <gibber/gibber-iq-helper.h>
 
 #include "salut-muc-channel.h"
+#include "tube-iface.h"
 
 G_BEGIN_DECLS
 
@@ -34,6 +36,7 @@ typedef struct _SalutTubesChannelClass SalutTubesChannelClass;
 
 struct _SalutTubesChannelClass {
     GObjectClass parent_class;
+    TpDBusPropertiesMixinClass dbus_props_class;
 };
 
 struct _SalutTubesChannel {
@@ -63,13 +66,29 @@ GType salut_tubes_channel_get_type (void);
   (G_TYPE_INSTANCE_GET_CLASS ((obj), SALUT_TYPE_TUBES_CHANNEL, \
                               SalutTubesChannelClass))
 
+void salut_tubes_channel_foreach (SalutTubesChannel *self,
+    TpExportableChannelFunc foreach, gpointer user_data);
+
 void salut_tubes_channel_close (SalutTubesChannel *channel);
 
 void salut_tubes_channel_bytestream_offered (SalutTubesChannel *chanel,
     GibberBytestreamIface *bytestream, GibberXmppStanza *msg);
 
-void tubes_message_received (SalutTubesChannel *channel, const gchar *sender,
-    GibberXmppStanza *stanza);
+void salut_tubes_channel_muc_message_received (SalutTubesChannel *channel,
+    const gchar *sender, GibberXmppStanza *stanza);
+
+void salut_tubes_channel_message_received (SalutTubesChannel *self,
+    const gchar *service, TpTubeType tube_type, TpHandle initiator_handle,
+    GHashTable *parameters, guint tube_id, guint portnum,
+    GibberXmppStanza *iq_req);
+
+void salut_tubes_channel_message_close_received (SalutTubesChannel *self,
+    TpHandle initiator_handle, guint tube_id);
+
+SalutTubeIface *salut_tubes_channel_tube_request (SalutTubesChannel *self,
+    const gchar *channel_type, const gchar *service);
+
+void salut_tubes_channel_send_iq_offer (SalutTubesChannel *self);
 
 G_END_DECLS
 
