@@ -28,6 +28,7 @@
 #include "salut-contact.h"
 #include "salut-contact-manager.h"
 
+#include <gibber/gibber-listener.h>
 #include <gibber/gibber-xmpp-connection-listener.h>
 #include <gibber/gibber-namespaces.h>
 #include <gibber/gibber-linklocal-transport.h>
@@ -35,7 +36,7 @@
 #define DEBUG_FLAG DEBUG_XCM
 #include "debug.h"
 
-#include "signals-marshal.h"
+#include "salut-signals-marshal.h"
 
 #define OUTGOING_CONNECTION_TIMEOUT 10000
 #define REFCOUNT_CONNECTION_TIMEOUT 10000
@@ -45,7 +46,7 @@ G_DEFINE_TYPE (SalutXmppConnectionManager, salut_xmpp_connection_manager, \
 
 static void
 new_connection_cb (GibberXmppConnectionListener *listener,
-    GibberXmppConnection *connection, struct sockaddr_storage *addr,
+    GibberXmppConnection *connection, struct sockaddr *addr,
     guint size, gpointer user_data);
 static gboolean
 create_new_outgoing_connection (SalutXmppConnectionManager *self,
@@ -102,13 +103,14 @@ struct _SalutXmppConnectionManagerPrivate
    * be allowed to request a new connection with the contact */
   /* GibberXmppConnection * -> SalutContact */
   GHashTable *connections_waiting_close;
+  gulong status_changed_id;
 
   gboolean dispose_has_run;
 };
 
 #define SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE(obj) \
     ((SalutXmppConnectionManagerPrivate *) \
-      ((SalutXmppConnectionManager *)obj)->priv)
+      ((SalutXmppConnectionManager *) obj)->priv)
 
 typedef struct
 {
@@ -547,8 +549,19 @@ connection_transport_disconnected_cb (GibberLLTransport *transport,
   DEBUG ("Connection transport disconnected");
   data.self = self;
   data.transport = transport;
+
+  /* remove_connection_having_transport() will unref GibberXmppConnection (the
+   * key of the hash) and unref SalutContact (the value of the hash). If
+   * SalutContact is freed at this step, this will unref SalutConnection, and
+   * may unref and free this SalutXmppConnectionManager. We don't want
+   * SalutXmppConnectionManager to be freed while we are still in the foreach
+   * loop. Hence the ref and unref of priv->connection, in order to free
+   * everything after the foreach loop.
+   *  */
+  g_object_ref (priv->connection);
   g_hash_table_foreach_remove (priv->connections,
       remove_connection_having_transport, &data);
+  g_object_unref (priv->connection);
 }
 
 static void
@@ -621,15 +634,14 @@ incoming_connection_found_contact (SalutXmppConnectionManager *self,
   g_signal_handlers_disconnect_matched (conn->transport,
      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
 
-  if (!gibber_ll_transport_get_address (
-        GIBBER_LL_TRANSPORT (conn->transport), &addr, &size))
+  if (!gibber_transport_get_sockaddr (conn->transport, &addr, &size))
     {
       DEBUG ("Failed to get address of connection from %s", contact->name);
       ret = FALSE;
       goto error;
     }
 
-  if (!salut_contact_has_address (contact, &addr))
+  if (!salut_contact_has_address (contact, (struct sockaddr *) &addr, size))
     {
       DEBUG ("Contact %s doesn't have that address", contact->name);
       ret = FALSE;
@@ -701,6 +713,7 @@ incoming_pending_connection_stream_opened_cb (GibberXmppConnection *conn,
   SalutXmppConnectionManagerPrivate *priv =
     SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE (self);
   GList *contacts;
+  guint contacts_len;
 
   DEBUG ("incoming pending connection with %s opened. Open it too", from);
   gibber_xmpp_connection_open (conn, from, priv->connection->name, "1.0");
@@ -720,14 +733,18 @@ incoming_pending_connection_stream_opened_cb (GibberXmppConnection *conn,
    * support that yet.
    * */
   if (from != NULL)
-    incoming_pending_connection_got_from (self, conn, from);
+    {
+      incoming_pending_connection_got_from (self, conn, from);
+      return;
+    }
 
   /* If it's a transport to just one contacts machine, hook it up right away.
    * This is needed because iChat doesn't send message with to and
    * from data...
    */
   contacts = g_hash_table_lookup (priv->incoming_pending_connections, conn);
-  if (g_list_length (contacts) == 1)
+  contacts_len = g_list_length (contacts);
+  if (contacts_len == 1)
     {
       SalutContact *contact = contacts->data;
 
@@ -735,6 +752,23 @@ incoming_pending_connection_stream_opened_cb (GibberXmppConnection *conn,
           "Assuming it's a connection from that contact", contact->name);
       incoming_connection_found_contact (self, conn, contact);
     }
+  else
+    {
+#ifdef ENABLE_DEBUG
+      GList *l;
+
+      DEBUG ("Incoming connection from a machine with %d contacts. "
+          "Can't assume its identity. Possible contacts are:",
+          contacts_len);
+
+      for (l = contacts; l != NULL; l = g_list_next (l))
+        {
+          SalutContact *contact = SALUT_CONTACT (l->data);
+
+          DEBUG ("--> %s\n", contact->name);
+        }
+#endif
+    }
 }
 
 static void
@@ -742,7 +776,7 @@ incoming_pending_connection_stanza_received_cb (GibberXmppConnection *conn,
                                                 GibberXmppStanza *stanza,
                                                 gpointer userdata)
 {
-  SalutXmppConnectionManager *self = SALUT_XMPP_CONNECTION_MANAGER (conn);
+  SalutXmppConnectionManager *self = SALUT_XMPP_CONNECTION_MANAGER (userdata);
   const gchar *from;
 
   /* If the identity wasn't clear from the stream opening we only wait to the
@@ -796,7 +830,7 @@ incoming_pending_connection_parse_error_cb (GibberXmppConnection *conn,
 static void
 new_connection_cb (GibberXmppConnectionListener *listener,
                    GibberXmppConnection *connection,
-                   struct sockaddr_storage *addr,
+                   struct sockaddr *addr,
                    guint size,
                    gpointer user_data)
 {
@@ -808,7 +842,7 @@ new_connection_cb (GibberXmppConnectionListener *listener,
   DEBUG("Handling new incoming connection");
 
   contacts = salut_contact_manager_find_contacts_by_address (
-      priv->contact_manager, addr);
+      priv->contact_manager, addr, size);
   if (contacts == NULL)
     {
       DEBUG ("Couldn't find a contact for the incoming connection");
@@ -850,18 +884,12 @@ disconnect_signals_foreach (gpointer key,
       0, 0, NULL, NULL, self);
 }
 
-void
-salut_xmpp_connection_manager_dispose (GObject *object)
+static void
+salut_xmpp_manager_close_all (SalutXmppConnectionManager *self)
 {
-  SalutXmppConnectionManager *self = SALUT_XMPP_CONNECTION_MANAGER (object);
   SalutXmppConnectionManagerPrivate *priv =
     SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE (self);
 
-  if (priv->dispose_has_run)
-    return;
-
-  priv->dispose_has_run = TRUE;
-
   /* Disconnect signals from all connections */
   g_hash_table_foreach (priv->connections, disconnect_signals_foreach, self);
   g_hash_table_foreach (priv->incoming_pending_connections,
@@ -899,12 +927,6 @@ salut_xmpp_connection_manager_dispose (GObject *object)
       priv->outgoing_pending_connections = NULL;
     }
 
-  if (priv->stanza_filters != NULL)
-    {
-      g_hash_table_destroy (priv->stanza_filters);
-      priv->stanza_filters = NULL;
-    }
-
   if (priv->connection_timers != NULL)
     {
       g_hash_table_destroy (priv->connection_timers);
@@ -926,6 +948,65 @@ salut_xmpp_connection_manager_dispose (GObject *object)
   free_stanza_filters_list (priv->all_connection_filters);
   priv->all_connection_filters = NULL;
 
+  if (priv->status_changed_id != 0)
+    {
+      g_signal_handler_disconnect (priv->connection, priv->status_changed_id);
+      priv->status_changed_id = 0;
+    }
+}
+
+static void
+connection_status_changed_cb (SalutConnection *conn,
+                              guint status,
+                              guint reason,
+                              SalutXmppConnectionManager *self)
+{
+  if (status == TP_CONNECTION_STATUS_DISCONNECTED)
+    {
+      salut_xmpp_manager_close_all (self);
+    }
+}
+
+static GObject *
+salut_xmpp_connection_manager_constructor (GType type,
+                                           guint n_props,
+                                           GObjectConstructParam *props)
+{
+  GObject *obj;
+  SalutXmppConnectionManager *self;
+  SalutXmppConnectionManagerPrivate *priv;
+
+  obj = G_OBJECT_CLASS (salut_xmpp_connection_manager_parent_class)->
+           constructor (type, n_props, props);
+
+  self = SALUT_XMPP_CONNECTION_MANAGER (obj);
+  priv = SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE (self);
+
+  priv->status_changed_id = g_signal_connect (priv->connection,
+      "status-changed", (GCallback) connection_status_changed_cb, self);
+
+  return obj;
+}
+
+static void
+salut_xmpp_connection_manager_dispose (GObject *object)
+{
+  SalutXmppConnectionManager *self = SALUT_XMPP_CONNECTION_MANAGER (object);
+  SalutXmppConnectionManagerPrivate *priv =
+    SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE (self);
+
+  if (priv->dispose_has_run)
+    return;
+
+  priv->dispose_has_run = TRUE;
+
+  if (priv->stanza_filters != NULL)
+    {
+      g_hash_table_destroy (priv->stanza_filters);
+      priv->stanza_filters = NULL;
+    }
+
+
   if (G_OBJECT_CLASS (salut_xmpp_connection_manager_parent_class)->dispose)
     G_OBJECT_CLASS (salut_xmpp_connection_manager_parent_class)->dispose (
         object);
@@ -991,6 +1072,7 @@ salut_xmpp_connection_manager_class_init (
   g_type_class_add_private (salut_xmpp_connection_manager_class,
       sizeof (SalutXmppConnectionManagerPrivate));
 
+  object_class->constructor = salut_xmpp_connection_manager_constructor;
   object_class->dispose = salut_xmpp_connection_manager_dispose;
 
   object_class->get_property = salut_xmpp_connection_manager_get_property;
@@ -1002,9 +1084,7 @@ salut_xmpp_connection_manager_class_init (
       "Salut Connection associated with this manager ",
       SALUT_TYPE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION,
       param_spec);
 
@@ -1015,9 +1095,7 @@ salut_xmpp_connection_manager_class_init (
       "manager",
       SALUT_TYPE_CONTACT_MANAGER,
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTACT_MANAGER,
       param_spec);
 
@@ -1085,14 +1163,19 @@ salut_xmpp_connection_manager_listen (SalutXmppConnectionManager *self,
     SALUT_XMPP_CONNECTION_MANAGER_GET_PRIVATE (self);
   int port;
 
-  for (port = 5298; port < 5400; port++)
+  /* The port 5298 is preferred to remain compatible with old versions of
+   * iChat. Try a few close to it, and if those fail, use a random port. */
+  for (port = 5298; port < 5300; port++)
     {
       GError *e = NULL;
-      if (gibber_xmpp_connection_listener_listen (priv->listener, port,
-            &e))
+      int ret;
+      ret = gibber_xmpp_connection_listener_listen (priv->listener, port, &e);
+
+      if (ret)
         break;
 
-      if (e->code != GIBBER_XMPP_CONNECTION_LISTENER_ERROR_ADDR_IN_USE)
+      if (!g_error_matches (e, GIBBER_LISTENER_ERROR,
+            GIBBER_LISTENER_ERROR_ADDRESS_IN_USE))
         {
           g_propagate_error (error, e);
           return -1;
@@ -1102,8 +1185,17 @@ salut_xmpp_connection_manager_listen (SalutXmppConnectionManager *self,
       e = NULL;
     }
 
-  if (port >= 5400)
-    return -1;
+  if (port >= 5300)
+    {
+      int ret = gibber_xmpp_connection_listener_listen (priv->listener, 0, error);
+
+      if (ret)
+        {
+          return gibber_xmpp_connection_listener_get_port (priv->listener);
+        }
+      else
+        return -1;
+    }
 
   return port;
 }
@@ -1250,7 +1342,7 @@ create_new_outgoing_connection (SalutXmppConnectionManager *self,
   GibberXmppConnection *connection;
   GibberLLTransport *transport;
   GArray *addrs;
-  gint i;
+  guint i;
   GError *e = NULL;
 
   DEBUG ("create a new outgoing connection to %s", contact->name);
@@ -1300,6 +1392,8 @@ create_new_outgoing_connection (SalutXmppConnectionManager *self,
            * open it */
           DEBUG ("waiting for remote contact (%s) open the connection",
               contact->name);
+          /* Don't ref connection, so it can be freed when removed from the
+           * hash table */
           g_hash_table_insert (priv->outgoing_pending_connections, connection,
               g_object_ref (contact));
 
@@ -1477,7 +1571,7 @@ salut_xmpp_connection_manager_reset_connection_timer (
     }
 }
 
-GSList *
+static GSList *
 find_filter (GSList *list,
              SalutXmppConnectionManagerStanzaFilterFunc filter_func,
              SalutXmppConnectionManagerStanzaCallbackFunc callback,
@@ -1500,7 +1594,7 @@ find_filter (GSList *list,
   return NULL;
 }
 
-GSList *
+static GSList *
 remove_filter (GSList *list,
                SalutXmppConnectionManagerStanzaFilterFunc filter_func,
                SalutXmppConnectionManagerStanzaCallbackFunc callback,
