@@ -23,6 +23,7 @@
 #include <glib-object.h>
 
 #include "salut-connection.h"
+#include "salut-tubes-channel.h"
 #include <gibber/gibber-muc-connection.h>
 #include <gibber/gibber-bytestream-iface.h>
 
@@ -60,11 +61,10 @@ GType salut_tube_dbus_get_type (void);
                               SalutTubeDBusClass))
 
 SalutTubeDBus *
-salut_tube_dbus_new (SalutConnection *conn, TpHandle handle,
-    TpHandleType handle_type, TpHandle self_handle,
+salut_tube_dbus_new (SalutConnection *conn, SalutTubesChannel *tubes_channel,
+    TpHandle handle, TpHandleType handle_type, TpHandle self_handle,
     GibberMucConnection *muc_connection, TpHandle initiator,
-    const gchar *service, GHashTable *parameters, guint id,
-    GibberBytestreamIface *bytestream);
+    const gchar *service, GHashTable *parameters, guint id);
 
 gboolean salut_tube_dbus_add_name (SalutTubeDBus *self, TpHandle handle,
     const gchar *name);
