@@ -19,7 +19,10 @@
 
 #include "tube-iface.h"
 
+#include <telepathy-glib/gtypes.h>
+
 #include "salut-connection.h"
+#include "salut-tubes-channel.h"
 
 #include <glib.h>
 
@@ -34,12 +37,40 @@ salut_tube_iface_accept (SalutTubeIface *self,
 }
 
 void
-salut_tube_iface_close (SalutTubeIface *self)
+salut_tube_iface_accepted (SalutTubeIface *self)
 {
   void (*virtual_method)(SalutTubeIface *) =
+    SALUT_TUBE_IFACE_GET_CLASS (self)->accepted;
+  if (virtual_method != NULL)
+    virtual_method (self);
+  /* else nothing to do */
+}
+
+gboolean
+salut_tube_iface_offer_needed (SalutTubeIface *self)
+{
+  gboolean (*virtual_method)(SalutTubeIface *) =
+    SALUT_TUBE_IFACE_GET_CLASS (self)->offer_needed;
+  g_assert (virtual_method != NULL);
+  return virtual_method (self);
+}
+
+int
+salut_tube_iface_listen (SalutTubeIface *self)
+{
+  gboolean (*virtual_method)(SalutTubeIface *) =
+    SALUT_TUBE_IFACE_GET_CLASS (self)->listen;
+  g_assert (virtual_method != NULL);
+  return virtual_method (self);
+}
+
+void
+salut_tube_iface_close (SalutTubeIface *self, gboolean closed_remotely)
+{
+  void (*virtual_method)(SalutTubeIface *, gboolean) =
     SALUT_TUBE_IFACE_GET_CLASS (self)->close;
   g_assert (virtual_method != NULL);
-  virtual_method (self);
+  virtual_method (self, closed_remotely);
 }
 
 void
@@ -67,6 +98,15 @@ salut_tube_iface_base_init (gpointer klass)
           "Salut connection object that owns this D-Bus tube object.",
           SALUT_TYPE_CONNECTION,
           G_PARAM_CONSTRUCT_ONLY |
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+      g_object_interface_install_property (klass, param_spec);
+
+      param_spec = g_param_spec_object (
+          "tubes-channel",
+          "SalutTubesChannel object",
+          "Salut tubes object that implements the old interface.",
+          SALUT_TYPE_TUBES_CHANNEL,
+          G_PARAM_CONSTRUCT_ONLY |
           G_PARAM_READWRITE |
           G_PARAM_STATIC_NAME |
           G_PARAM_STATIC_NICK |
@@ -80,10 +120,7 @@ salut_tube_iface_base_init (gpointer klass)
           "owns this D-Bus tube object.",
           0, G_MAXUINT32, 0,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
@@ -93,10 +130,7 @@ salut_tube_iface_base_init (gpointer klass)
           "that owns this D-Bus tube object.",
           0, G_MAXUINT32, 0,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
@@ -106,10 +140,7 @@ salut_tube_iface_base_init (gpointer klass)
           "connection's self handle if our handle is a room handle.",
           0, G_MAXUINT, 0,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
@@ -118,10 +149,7 @@ salut_tube_iface_base_init (gpointer klass)
           "The unique identifier of this tube",
           0, G_MAXUINT32, 0,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
@@ -129,22 +157,16 @@ salut_tube_iface_base_init (gpointer klass)
           "Tube type",
           "The TpTubeType this D-Bus tube object.",
           0, G_MAXUINT32, 0,
-          G_PARAM_READABLE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
-          "initiator",
+          "initiator-handle",
           "Initiator handle",
-          "The TpHandle of the initiator of this D-Bus tube object.",
+          "The TpHandle of the initiator of tube object.",
           0, G_MAXUINT32, 0,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_string (
@@ -153,22 +175,16 @@ salut_tube_iface_base_init (gpointer klass)
           "the service associated with this D-BUS tube object.",
           "",
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_boxed (
           "parameters",
           "parameters GHashTable",
           "GHashTable containing parameters of this DBUS tube object.",
-          G_TYPE_HASH_TABLE,
+          TP_HASH_TYPE_STRING_VARIANT_MAP,
           G_PARAM_CONSTRUCT_ONLY |
-          G_PARAM_READWRITE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       param_spec = g_param_spec_uint (
@@ -176,10 +192,7 @@ salut_tube_iface_base_init (gpointer klass)
           "Tube state",
           "The SalutTubeState of this DBUS tube object",
           0, G_MAXUINT32, TP_TUBE_STATE_REMOTE_PENDING,
-          G_PARAM_READABLE |
-          G_PARAM_STATIC_NAME |
-          G_PARAM_STATIC_NICK |
-          G_PARAM_STATIC_BLURB);
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
       g_object_interface_install_property (klass, param_spec);
 
       initialized = TRUE;
